@@ -15,7 +15,7 @@ anyscale_dir = os.path.dirname(os.path.abspath(__file__))
 path.append(os.path.join(anyscale_dir, "client"))
 path.append(os.path.join(anyscale_dir, "sdk"))
 
-from anyscale import experimental_integrations
+from anyscale import integrations
 from anyscale.cluster import get_job_submission_client_cluster_info  # noqa: E402
 from anyscale.connect import ClientBuilder  # noqa: E402
 from anyscale.sdk.anyscale_client.sdk import AnyscaleSDK  # noqa: E402
@@ -40,7 +40,7 @@ for attr, _ in inspect.getmembers(ClientBuilder, inspect.isfunction):
 
         new_session_builder.__name__ = attr
         new_session_builder.__doc__ = target.__doc__ + BUILDER_HELP_FOOTER
-        setattr(new_session_builder, "__signature__", inspect.signature(target))
+        new_session_builder.__signature__ = inspect.signature(target)  # type: ignore
 
         return new_session_builder
 
