@@ -16,11 +16,11 @@ from anyscale.api import (
     format_api_exception,
 )
 from anyscale.cli_logger import BlockLogger
-import anyscale.client.openapi_client as openapi_client
+from anyscale.client import openapi_client
 from anyscale.client.openapi_client.api.default_api import DefaultApi
 from anyscale.client.openapi_client.rest import ApiException as ApiExceptionInternal
 import anyscale.conf
-import anyscale.sdk.anyscale_client as anyscale_client
+from anyscale.sdk import anyscale_client
 from anyscale.sdk.anyscale_client.api.default_api import DefaultApi as AnyscaleApi
 import anyscale.shared_anyscale_utils.conf as shared_anyscale_conf
 
@@ -38,7 +38,7 @@ class AuthenticationBlock:
     object.
     """
 
-    def __init__(
+    def __init__(  # noqa: PLR0913
         self,
         cli_token: Optional[str] = None,
         host: Optional[str] = None,
@@ -261,7 +261,7 @@ class AuthenticationBlock:
 _auth_api_client: Optional[Union[Exception, AuthenticationBlock]] = None
 
 
-def get_auth_api_client(
+def get_auth_api_client(  # noqa: PLR0913
     cli_token: Optional[str] = None,
     host: Optional[str] = None,
     use_asyncio: bool = False,
