@@ -70,6 +70,14 @@ def compare_dicts_diff(d1: Dict[Any, Any], d2: Dict[Any, Any]) -> str:
     )
 
 
+def log_resource_not_found_error(
+    resource_name: str, resource_id: str, logger: BlockLogger
+) -> None:
+    logger.error(
+        f"Could not find {resource_name} with id {resource_id}. Please validate that you're using the correct AWS account/credentials and that the resource values are correct"
+    )
+
+
 def verify_aws_vpc(
     cloud_resource: CreateCloudResource,
     boto3_session: boto3.Session,
@@ -89,7 +97,7 @@ def verify_aws_vpc(
         vpc.load()
     except ClientError as e:
         if e.response["Error"]["Code"] == "InvalidVpcID.NotFound":
-            logger.error(f"VPC with id {cloud_resource.aws_vpc_id} does not exist.")
+            log_resource_not_found_error("VPC", cloud_resource.aws_vpc_id, logger)
             return False
         raise e
 
@@ -169,7 +177,7 @@ def verify_aws_subnets(  # noqa: PLR0911
     for subnet, subnet_id in zip(subnets, subnet_ids):
         # Verify subnet exists
         if not subnet:
-            logger.error(f"Subnet with id {subnet_id} does not exist.")
+            log_resource_not_found_error("Subnet", subnet_id, logger)
             return False
 
         # Verify the Subnet has "enough" capacity.
@@ -373,8 +381,8 @@ def verify_aws_security_groups(
         anyscale_security_group.load()
     except ClientError as e:
         if e.response["Error"]["Code"] == "InvalidGroup.NotFound":
-            logger.error(
-                f"Security group with id {anyscale_security_group_arn} does not exist."
+            log_resource_not_found_error(
+                "Security group", anyscale_security_group_arn, logger
             )
             return False
         raise e
@@ -445,7 +453,7 @@ def verify_aws_s3(
     # Check for the existence of `creation_date` because this incurs a `list_bucket` call.
     # Calling `.load()` WILL NOT ERROR in cases where the caller does not have access to the bucket.
     if s3_bucket.creation_date is None:
-        logger.error(f"S3 object with id {cloud_resource.aws_s3_id} does not exist.")
+        log_resource_not_found_error("S3 bucket", cloud_resource.aws_s3_id, logger)
         return False
 
     has_correct_cors_rule = False
@@ -559,12 +567,12 @@ def verify_aws_efs(  # noqa: PLR0911, PLR0912
         )
     except ClientError as e:
         if e.response["Error"]["Code"] == "FileSystemNotFound":
-            logger.error(f"EFS with id {cloud_resource.aws_efs_id} does not exist.")
+            log_resource_not_found_error("EFS", cloud_resource.aws_efs_id, logger)
             return False
         raise e
 
     if len(file_systems_response.get("FileSystems", [])) == 0:
-        logger.error(f"EFS with id {cloud_resource.aws_efs_id} does not exist.")
+        log_resource_not_found_error("EFS", cloud_resource.aws_efs_id, logger)
         return False
 
     # verify that there is a mount target for each subnet and security group
@@ -620,8 +628,10 @@ def verify_aws_cloudformation_stack(
         stack.load()
     except ClientError as e:
         if e.response["Error"]["Code"] == "ValidationError":
-            logger.error(
-                f"CloudFormation stack with id {cloud_resource.aws_cloudformation_stack_id} does not exist."
+            log_resource_not_found_error(
+                "CloudFormation stack",
+                cloud_resource.aws_cloudformation_stack_id,
+                logger,
             )
             return False
         raise e
