@@ -6,7 +6,7 @@ from typing import Optional
 
 import click
 
-from anyscale.api import get_anyscale_api_client
+from anyscale.authenticate import get_auth_api_client
 from anyscale.formatters import common_formatter
 from anyscale.sdk.anyscale_client import CreateSessionCommand
 
@@ -33,7 +33,7 @@ def list_session_commands(
 ) -> None:
     """Lists commands that have been executed on Session with SESSION_ID"""
 
-    api_client = get_anyscale_api_client()
+    api_client = get_auth_api_client().anyscale_api_client
     response = api_client.list_session_commands(
         session_id, count=count, paging_token=paging_token
     )
@@ -46,7 +46,7 @@ def list_session_commands(
 def get_session_command(session_command_id: str) -> None:
     """Get details about the Session Command with id SESSION_COMMAND_ID"""
 
-    api_client = get_anyscale_api_client()
+    api_client = get_auth_api_client().anyscale_api_client
     response = api_client.get_session_command(session_command_id)
 
     print(common_formatter.prettify_json(response.to_dict()))
@@ -57,7 +57,7 @@ def get_session_command(session_command_id: str) -> None:
 def kill_session_command(session_command_id: str) -> None:
     """Kills the Session Command with id SESSION_COMMAND_ID"""
 
-    api_client = get_anyscale_api_client()
+    api_client = get_auth_api_client().anyscale_api_client
     response = api_client.kill_session_command(session_command_id)
 
     print(common_formatter.prettify_json(response.to_dict()))
@@ -71,7 +71,7 @@ def kill_session_command(session_command_id: str) -> None:
 def create_session_command(session_id: str, shell_command: str) -> None:
     """Creates and executes SHELL_COMMAND on the session with id SESSION_ID."""
 
-    api_client = get_anyscale_api_client()
+    api_client = get_auth_api_client().anyscale_api_client
     create_data = CreateSessionCommand(
         session_id=session_id, shell_command=shell_command
     )
