@@ -139,7 +139,7 @@ def setup_cloud(  # noqa: PLR0913
     vpc_peering_target_vpc_id: Optional[str],
 ) -> None:
 
-    if anyscale_managed:
+    if provider == "aws":  # only provides Anyscale Managed option for AWS
         if (
             vpc_peering_ip_range
             or vpc_peering_target_vpc_id
@@ -155,18 +155,27 @@ def setup_cloud(  # noqa: PLR0913
             yes=yes,
             folder_id=gcp_folder_id,
         )
-    else:
-        CloudController().setup_cloud(
-            provider=provider,
-            region=region,
-            name=name,
-            yes=yes,
-            gce=gce,
-            folder_id=gcp_folder_id,
-            vpc_peering_ip_range=vpc_peering_ip_range,
-            vpc_peering_target_project_id=vpc_peering_target_project_id,
-            vpc_peering_target_vpc_id=vpc_peering_target_vpc_id,
-        )
+    elif provider == "gcp":
+        if anyscale_managed:
+            CloudController().setup_managed_cloud(
+                provider=provider,
+                region=region,
+                name=name,
+                yes=yes,
+                folder_id=gcp_folder_id,
+            )
+        else:
+            CloudController().setup_cloud(
+                provider=provider,
+                region=region,
+                name=name,
+                yes=yes,
+                gce=gce,
+                folder_id=gcp_folder_id,
+                vpc_peering_ip_range=vpc_peering_ip_range,
+                vpc_peering_target_project_id=vpc_peering_target_project_id,
+                vpc_peering_target_vpc_id=vpc_peering_target_vpc_id,
+            )
 
 
 @cloud_cli.command(
