@@ -136,7 +136,7 @@ def cluster_cli() -> None:
         "the same Anyscale cloud and will not require an authentication token."
     ),
 )
-def start(
+def start(  # noqa: PLR0913
     name: Optional[str],
     env: Optional[str],
     docker: Optional[str],
@@ -351,7 +351,7 @@ def archive(
     help="Max items to show in list.",
     callback=validate_non_negative_arg,
 )
-def list(
+def list(  # noqa: A001, PLR0913
     name: Optional[str],
     cluster_id: Optional[str],
     project_id: Optional[str],
