@@ -130,7 +130,7 @@ def archive_cluster_compute(
     help="Max items to show in list.",
     callback=validate_non_negative_arg,
 )
-def list(
+def list(  # noqa: A001
     name: Optional[str],
     cluster_compute_id: Optional[str],
     include_shared: bool,
