@@ -60,7 +60,7 @@ from anyscale.controllers.exec_controller import ExecController
     default=False,
 )
 @click.argument("commands", nargs=-1, type=str)
-def anyscale_exec(
+def anyscale_exec(  # noqa: PLR0913
     session_name: str,
     cluster_name: Optional[str],
     screen: bool,
