@@ -44,7 +44,7 @@ def job_cli() -> None:
 @click.argument(
     "entrypoint", required=False, nargs=-1,
 )
-def submit(
+def submit(  # noqa: PLR0913
     job_config_file: str,
     entrypoint: List[str],
     name: Optional[str],
@@ -70,7 +70,7 @@ def submit(
     """
 
     job_controller = JobController()
-    id = job_controller.submit(
+    id = job_controller.submit(  # noqa: A001
         job_config_file,
         name,
         description,
@@ -114,7 +114,7 @@ def submit(
     help="Max items to show in list.",
     callback=validate_non_negative_arg,
 )
-def list(
+def list(  # noqa: A001, PLR0913
     name: Optional[str],
     job_id: Optional[str],
     project_id: Optional[str],
@@ -163,21 +163,9 @@ def terminate(job_id: Optional[str], name: Optional[str]) -> None:
 )
 def logs(job_id: Optional[str], name: Optional[str], follow: bool = False,) -> None:
     job_controller = JobController()
-    try:
-        if follow:
-            job_controller.stream_logs(job_id=job_id, job_name=name)
-            return
-    except Exception as e:
-        log.info(f"Job logs could not be followed due to error: {e}")
-    try:
-        log.info("Now downloading logs from S3...")
-        job_controller.download_logs(job_id=job_id, job_name=name)
-    except Exception as e:
-        log.info(f"Job logs could not be downloaded due to error: {e}")
-        log.info("Now trying default method of fetching logs.")
-        job_controller.logs(
-            job_id=job_id, job_name=name, should_follow=follow,
-        )
+    job_controller.logs(
+        job_id=job_id, job_name=name, should_follow=follow,
+    )
 
 
 @job_cli.command(name="wait")
