@@ -41,7 +41,7 @@ def list_clouds(show_json: bool) -> None:
 )
 @click.option("--json", "show_json", help="Return the results in json", is_flag=True)
 @click.pass_context
-def project_list(ctx: Any, show_json: bool) -> None:
+def project_list(ctx: Any, show_json: bool) -> None:  # noqa: ARG001
     log.warning(
         "`anyscale list projects` has been deprecated. Please use `anyscale project list` instead."
     )
@@ -64,7 +64,7 @@ def project_list(ctx: Any, show_json: bool) -> None:
 )
 @click.option("--all", help="List all clusters, including inactive ones.", is_flag=True)
 @click.option("--json", "show_json", help="Return the results in json", is_flag=True)
-def session_list(name: Optional[str], all: bool, show_json: bool) -> None:
+def session_list(name: Optional[str], all: bool, show_json: bool) -> None:  # noqa: A002
     log.warning(
         "`anyscale list sessions` has been deprecated. Please use `anyscale cluster list` instead."
     )
