@@ -38,6 +38,10 @@ def deploy(
     healthcheck_url: Optional[str],
 ) -> None:
     # TODO[Bruce]: Remove update flag once fully test apply function.
+    """Deploy a service (create or update).
+
+    This is *only* supported for v1 services.
+    """
     service_controller = ServiceController()
     service_controller.deploy(
         service_config_file,
@@ -55,7 +59,7 @@ def deploy(
 )
 @click.option(
     "-f",
-    "--service_config_file",
+    "--service-config-file",
     required=True,
     help="The path of the service configuration file.",
 )
@@ -74,6 +78,10 @@ def rollout(
     version: Optional[str],
     canary_percent: Optional[int],
 ) -> None:
+    """Start or update a service rollout to a new version.
+
+    This is *only* supported for v2 services.
+    """
     service_controller = ServiceController()
     service_controller.rollout(
         service_config_file, name=name, version=version, canary_percent=canary_percent
@@ -121,6 +129,10 @@ def list(  # noqa: A001, PLR0913
     include_archived: bool,
     max_items: int,
 ) -> None:
+    """List services based on the provided filters.
+
+    This returns both v1 and v2 services.
+    """
     service_controller = ServiceController()
     service_controller.list(
         name=name,
@@ -133,30 +145,68 @@ def list(  # noqa: A001, PLR0913
 
 
 @service_cli.command(name="archive", help="Archive a service.")
-@click.option("--service-id", "--id", required=False, help="Id of service.")
+@click.option("--service-id", "--id", required=False, help="ID of service.")
 @click.option("--name", "-n", required=False, help="Name of service.")
-def archive(service_id: Optional[str], name: Optional[str]) -> None:
+@click.option(
+    "--service-config-file", "-f", help="The path of the service configuration file.",
+)
+def archive(
+    service_id: Optional[str], name: Optional[str], service_config_file: Optional[str]
+) -> None:
+    """Archive a service, which must already be terminated.
+
+    This is currently only supported for v1 services but should be extended to v2.
+    """
     service_controller = ServiceController()
-    service_controller.archive(service_id=service_id, service_name=name)
+    service_id = service_controller.get_service_id(
+        service_id, name, service_config_file
+    )
+    service_controller.archive(service_id)
 
 
 @service_cli.command(
-    name="rollback", help="Attempt to rollback a service asynchronously."
+    name="rollback", help="Attempt to rollback a v2 service asynchronously."
 )
-@click.option("--service-id", "--id", required=True, help="Id of service.")
-def rollback(service_id: str) -> None:
-    # TODO (kamenshah)
-    # Add service_name and service_config_file as arguments
-    # If service_id is None then derive id from service_name or service_config_file
+@click.option(
+    "--service-id", "--id", default=None, help="ID of service.",
+)
+@click.option("--name", "-n", required=False, default=None, help="Name of service.")
+@click.option(
+    "--service-config-file", "-f", help="The path of the service configuration file.",
+)
+def rollback(
+    service_id: Optional[str], name: Optional[str], service_config_file: Optional[str]
+) -> None:
+    """Perform a rollback for a service that is currently in a rollout.
+
+    This *only* applies to v2 services.
+    """
     service_controller = ServiceController()
-    service_controller.rollback(service_id=service_id,)
+    service_id = service_controller.get_service_id(
+        service_id, name, service_config_file
+    )
+    service_controller.rollback(service_id)
 
 
 @service_cli.command(
     name="terminate", help="Attempt to terminate a service asynchronously."
 )
-@click.option("--service-id", "--id", required=False, help="Id of service.")
+@click.option(
+    "--service-id", "--id", required=False, help="ID of service.",
+)
 @click.option("--name", "-n", required=False, help="Name of service.")
-def service(service_id: Optional[str], name: Optional[str]) -> None:
+@click.option(
+    "--service-config-file", "-f", help="The path of the service configuration file.",
+)
+def terminate(
+    service_id: Optional[str], name: Optional[str], service_config_file: Optional[str]
+) -> None:
+    """Terminate a service.
+
+    This applies to both v1 and v2 services.
+    """
     service_controller = ServiceController()
-    service_controller.terminate(service_id=service_id, service_name=name)
+    service_id = service_controller.get_service_id(
+        service_id, name, service_config_file
+    )
+    service_controller.terminate(service_id)
