@@ -1,5 +1,6 @@
 import json
 import os
+import sys
 from typing import Any
 
 import click
@@ -60,7 +61,7 @@ def workspace_cli() -> None:
     help=("Ray version for the custom docker image."),
 )
 @click.option("--compute-config-id", required=True)
-def create(
+def create(  # noqa: PLR0913
     name: str,
     project_id: str,
     cloud_id: str,
@@ -194,7 +195,7 @@ def activate(name: str) -> None:
     try:
         workspace = get_workspace_from_name(name)
         workspace_id = workspace.id
-    except Exception:
+    except Exception:  # noqa: BLE001
         workpaces_url = get_endpoint("/workspaces")
         raise click.ClickException(
             f"There is no workspace {name} registered with Anyscale. You can view your workspaces here: {workpaces_url}"
@@ -362,16 +363,16 @@ def get_workspace_from_name(name: str) -> Any:
         name=name
     ).results
     if len(results) == 0:
-        _exit_error("No workspace with name {} found.".format(name))
+        _exit_error(f"No workspace with name {name} found.")
     elif len(results) > 1:
-        _exit_error("Multiple workspaces with name {} found.".format(name))
+        _exit_error(f"Multiple workspaces with name {name} found.")
     return results[0]
 
 
 def _exit_error(msg: str) -> None:
     print()
     print(msg)
-    exit(1)
+    sys.exit(1)
 
 
 def _execute_shell_command(cluster_id: str, shell_command: str):
