@@ -56,8 +56,8 @@ BUILD_STEPS = [
 def _get_base_image(image: str, ray_version: str, cpu_or_gpu: str) -> str:
     py_version = "".join(str(x) for x in sys.version_info[0:2])
     if py_version not in ["36", "37", "38"]:
-        raise ValueError("No default docker image for py{}".format(py_version))
-    return "anyscale/{}:{}-py{}-{}".format(image, ray_version, py_version, cpu_or_gpu)
+        raise ValueError(f"No default docker image for py{py_version}")
+    return f"anyscale/{image}:{ray_version}-py{py_version}-{cpu_or_gpu}"
 
 
 class PrepareClusterAction(str, Enum):
@@ -69,7 +69,7 @@ class PrepareClusterAction(str, Enum):
 
 
 class PrepareClusterBlock:
-    def __init__(
+    def __init__(  # noqa: PLR0913
         self,
         project_id: str,
         cluster_name: Optional[str],
@@ -236,7 +236,7 @@ class PrepareClusterBlock:
 
         self.log.close_block(self.block_label)
 
-    def _start_or_create_cluster(  # noqa: C901
+    def _start_or_create_cluster(  # noqa: C901, PLR0913
         self,
         project_id: str,
         build_id: str,
@@ -273,7 +273,7 @@ class PrepareClusterBlock:
             self.log.debug("-> Starting a new cluster")
             used_names = [s.name for s in results]
             for i in range(MAX_CLUSTERS):
-                name = "cluster-{}".format(i)
+                name = f"cluster-{i}"
                 if name not in used_names:
                     cluster_name = name
                     self.log.debug("Starting cluster", cluster_name)
@@ -306,7 +306,7 @@ class PrepareClusterBlock:
             allow_public_internet_traffic,
         )
 
-    def _start_cluster_if_required(
+    def _start_cluster_if_required(  # noqa: PLR0913
         self,
         cluster: Session,
         start_required: bool,
@@ -411,7 +411,7 @@ class PrepareClusterBlock:
 
         self.log.info(f"{left_pad}{'link:': <30}{url}", block_label=self.block_label)
 
-    def _create_or_update_session_data(
+    def _create_or_update_session_data(  # noqa: PLR0913
         self,
         cluster_name: Optional[str],
         project_id: str,
@@ -433,7 +433,7 @@ class PrepareClusterBlock:
 
         if not cluster_name:
             # Generate a cluster name
-            cluster_name = "cluster-{}".format(generate_slug())
+            cluster_name = f"cluster-{generate_slug()}"
 
         cluster = get_cluster(self.anyscale_api_client, project_id, cluster_name)
 
@@ -441,7 +441,7 @@ class PrepareClusterBlock:
             while cluster:
                 # User wants to create a new cluster but cluster with name already exists,
                 # generate a new name until we have one that's not taken.
-                cluster_name = "cluster-{}".format(generate_slug())
+                cluster_name = f"cluster-{generate_slug()}"
                 cluster = get_cluster(
                     self.anyscale_api_client, project_id, cluster_name
                 )
@@ -561,7 +561,7 @@ class PrepareClusterBlock:
                 f"py{py_version}", ray_version
             ).result
             return build
-        except Exception:
+        except Exception:  # noqa: BLE001
             raise RuntimeError(
                 f"Failed to get default cluster env for Ray: {ray_version} on Python: py{py_version}"
             )
@@ -639,7 +639,7 @@ class PrepareClusterBlock:
         assert build_to_use  # for mypy
         return build_to_use
 
-    def _build_cluster_env_if_needed(
+    def _build_cluster_env_if_needed(  # noqa: PLR0913
         self,
         project_id: str,
         build_pr: Optional[int],
@@ -695,7 +695,7 @@ class PrepareClusterBlock:
             config_name += f"-{build_commit}"
         # Force creation of a unique app config.
         if force_rebuild:
-            config_name += "-{}".format(int(time.time()))
+            config_name += f"-{int(time.time())}"
         app_templates = list_entities(
             self.anyscale_api_client.list_app_configs,
             filters={"name_contains": config_name},
@@ -705,14 +705,14 @@ class PrepareClusterBlock:
             build_steps = BUILD_STEPS.copy()
             # Add a unique command to bust the Makisu cache if needed.
             # Otherwise we could end up caching a previous fetch.
-            build_steps.append("echo UNIQUE_ID={}".format(config_name))
+            build_steps.append(f"echo UNIQUE_ID={config_name}")
             if build_pr:
                 build_steps.append(
                     "cd ray && git fetch origin pull/{}/head:target && "
                     "git checkout target".format(build_pr)
                 )
             if build_commit:
-                build_steps.append("cd ray && git checkout {}".format(build_commit))
+                build_steps.append(f"cd ray && git checkout {build_commit}")
             build_steps.append(
                 'cd ray/python && sudo env "PATH=$PATH" python setup.py develop'
             )
@@ -789,7 +789,7 @@ class PrepareClusterBlock:
         return compute_template_id
 
     def _get_cluster_compute_id_from_name(
-        self, project_id: str, cluster_compute_name: str
+        self, project_id: str, cluster_compute_name: str  # noqa: ARG002
     ) -> str:
         """Gets the cluster compute id given a cluster name."""
         cluster_computes = self.api_client.search_compute_templates_api_v2_compute_templates_search_post(
@@ -813,7 +813,7 @@ class PrepareClusterBlock:
         Register compute template with a default name and return the compute template id."""
         created_template = self.api_client.create_compute_template_api_v2_compute_templates_post(
             create_compute_template=CreateComputeTemplate(
-                name="autogenerated-config-{}".format(datetime.now().isoformat()),
+                name=f"autogenerated-config-{datetime.now().isoformat()}",
                 project_id=project_id,
                 config=config_object,
                 anonymous=True,
@@ -836,7 +836,7 @@ class PrepareClusterBlock:
                 cluster_compute_2_id
             ).result.config
             return bool(cluster_compute_1 == cluster_compute_2)
-        except Exception as e:
+        except Exception as e:  # noqa: BLE001
             self.log.debug(f"Error comparing cluster compute: {e}")
             return False
 
@@ -871,7 +871,7 @@ class PrepareClusterBlock:
                     self.api_client, cloud_id=organization.default_cloud_id
                 )
                 return str(cloud_name)
-            except Exception:
+            except Exception:  # noqa: BLE001
                 return None
         return None
 
@@ -890,7 +890,7 @@ class PrepareClusterBlock:
         if cloud_id:
             try:
                 cloud = self.anyscale_api_client.get_cloud(cloud_id).result
-            except Exception:
+            except Exception:  # noqa: BLE001
                 raise RuntimeError(f"Failed to fetch Cloud with id: {cloud_id}.")
         else:
             clouds = self._get_all_clouds()
@@ -904,10 +904,8 @@ class PrepareClusterBlock:
 
         cloud_name = cloud.name
         self.log.debug(
-            (
-                f"Using last active cloud '{cloud_name}'. "
-                "Call anyscale.cloud('...').connect() to overwrite."
-            )
+            f"Using last active cloud '{cloud_name}'. "
+            "Call anyscale.cloud('...').connect() to overwrite."
         )
         return cast(str, cloud_name)
 
@@ -1012,7 +1010,7 @@ class PrepareClusterBlock:
         return update_required
 
 
-def create_prepare_cluster_block(
+def create_prepare_cluster_block(  # noqa: PLR0913
     project_id: str,
     cluster_name: Optional[str],
     autosuspend_timeout: int,
