@@ -186,18 +186,7 @@ class CloudController(BaseController):
         """
         Sets up a cloud provider
         """
-        if provider == "aws":
-            # If the region is blank, change it to the default for AWS.
-            if region is None:
-                region = "us-west-2"
-            regions_available = get_available_regions()
-            if region not in regions_available:
-                raise ClickException(
-                    f"Region '{region}' is not available. Regions available are: "
-                    f"{', '.join(map(repr, regions_available))}"
-                )
-            self.setup_aws(region=region, name=name, yes=yes)
-        elif provider == "gcp":
+        if provider == "gcp":
             # If the region is blank, change it to the default for GCP.
             if region is None:
                 region = "us-west1"
@@ -244,7 +233,7 @@ class CloudController(BaseController):
             )
         else:
             raise ClickException(
-                f"Invalid Cloud provider: {provider}. Available providers are [aws, gcp]."
+                f'Invalid Cloud provider: {provider}; only "gcp" is supported.'
             )
 
     def run_cloudformation(
@@ -501,26 +490,6 @@ class CloudController(BaseController):
                 return
             self.log.info("AWS cross account roles deletion complete.")
 
-    def setup_aws(self, region: str, name: str, yes: bool = False) -> None:
-
-        confirm(
-            "\nYou are about to give Anyscale access to EC2 permissions necessary to manage clusters.\n"
-            "A separate AWS role is created for your clusters to run with \nand will be granted readonly access to ECR & S3 Full Access in your AWS account.\n\n"
-            "Continue?",
-            yes,
-        )
-
-        self.setup_aws_cross_account_role(region, name)
-        self.setup_aws_ray_role(region, DEFAULT_RAY_IAM_ROLE)
-
-        self.log.info("AWS credentials setup complete.")
-        self.log.info(
-            "You can revoke the access at any time by deleting anyscale IAM user/role in your account."
-        )
-        self.log.info(
-            "Head over to the web UI to create new sessions in your AWS account."
-        )
-
     def setup_aws_cross_account_role(self, region: str, name: str) -> None:
         response = (
             self.api_client.get_anyscale_aws_account_api_v2_clouds_anyscale_aws_account_get()
