@@ -1,5 +1,6 @@
 from abc import ABC, abstractmethod
 import asyncio
+import contextlib
 from datetime import datetime, timedelta
 import json
 import os
@@ -9,7 +10,8 @@ import traceback
 from typing import Any, cast, Dict, List, Optional, Tuple, Type, Union
 from urllib.parse import urlparse
 
-from aiohttp import ClientSession, WSMsgType
+from aiohttp import ClientResponseError, ClientSession, WSMsgType
+from aiohttp.web import HTTPNotFound
 import click
 from pydantic import BaseModel, Field, root_validator
 import requests
@@ -96,7 +98,7 @@ def _validate_conda_option(conda_option: Union[str, Dict]) -> Union[str, Dict]:
                 raise click.ClickException(f"Can't find conda YAML file {yaml_file}.")
             try:
                 result = yaml.safe_load(yaml_file.read_text())
-            except Exception as e:
+            except Exception as e:  # noqa: BLE001
                 raise click.ClickException(
                     f"Failed to read conda file {yaml_file}: {e}."
                 )
@@ -153,12 +155,14 @@ def _working_dir_is_remote_uri(working_dir: str) -> bool:
 
 def _validate_working_dir(working_dir_option: str) -> str:
     """If working_dir is a local directory, check that it exists."""
-    if not _working_dir_is_remote_uri(working_dir_option):
-        # We have been given a path to a local directory.
-        if not Path(working_dir_option).is_dir():
-            raise click.ClickException(
-                f"working_dir {working_dir_option} is not a valid local directory or remote URI."
-            )
+    # We have been given a path to a local directory.
+    if (
+        not _working_dir_is_remote_uri(working_dir_option)
+        and not Path(working_dir_option).is_dir()
+    ):
+        raise click.ClickException(
+            f"working_dir {working_dir_option} is not a valid local directory or remote URI."
+        )
 
     return working_dir_option
 
@@ -186,7 +190,7 @@ def _validate_upload_path(upload_path: str) -> str:
     """Check that the upload path is a valid S3 or GS remote URI."""
     try:
         parsed_upload_path = urlparse(upload_path)
-    except Exception as e:
+    except Exception as e:  # noqa: BLE001
         raise click.ClickException(
             f"Failed to parse `upload_path` {upload_path} as a URI (e.g. 's3://my_bucket/my_dir'): {e}"
         )
@@ -211,17 +215,11 @@ def _validate_env_vars(env_vars: Dict[str, str]) -> Dict[str, str]:
     return env_vars
 
 
-class JobConfig(BaseModel):
+class BaseHAJobConfig(BaseModel):
     """
-    Job Config model for CLI. Validate and preprocess so `entrypoint`, `runtime_env_config`,
-    `build_id`, `compute_config_id`, `max_retries` have the correct values to call
-    `/api/v2/decorated_ha_jobs/create`.
+    Base job configuration for both Production Jobs and Services.
     """
 
-    entrypoint: str = Field(
-        ...,
-        description="A script that will be run to start your job. This command will be run in the root directory of the specified runtime env. Eg. 'python script.py'",
-    )
     name: Optional[str] = Field(
         None,
         description="Name of job to be submitted. Default will be used if not provided.",
@@ -389,7 +387,7 @@ class JobConfig(BaseModel):
         return values
 
     @root_validator
-    def validate_runtime_env(cls: Any, values: Any) -> Any:  # noqa: N805
+    def validate_runtime_env(cls: Any, values: Any) -> Any:  # noqa: N805, PLR0912
         runtime_env = values.get("runtime_env")
         if runtime_env is not None:
             if "conda" in runtime_env:
@@ -442,6 +440,19 @@ class JobConfig(BaseModel):
         return values
 
 
+class JobConfig(BaseHAJobConfig):
+    """
+    Job Config model for CLI. Validate and preprocess so `entrypoint`, `runtime_env_config`,
+    `build_id`, `compute_config_id`, `max_retries` have the correct values to call
+    `/api/v2/decorated_ha_jobs/create`.
+    """
+
+    entrypoint: str = Field(
+        ...,
+        description="A script that will be run to start your job. This command will be run in the root directory of the specified runtime env. Eg. 'python script.py'",
+    )
+
+
 class LogProvider(ABC):
     @abstractmethod
     def __init__(self, dns_name: str, anyscale_token: str, query: str) -> None:
@@ -457,10 +468,10 @@ class LogProvider(ABC):
     def close(self) -> None:
         """Close any connection this log provider uses"""
 
-    def __enter__(self, *args, **kwargs):
+    def __enter__(self, *args, **kwargs):  # noqa: B027
         pass
 
-    def __exit__(self, *args, **kwargs):
+    def __exit__(self, *args, **kwargs):  # noqa: ARG002
         self.close()
 
 
@@ -477,7 +488,7 @@ class LokiLogProvider(LogProvider):
             "direction": "forward",
         }
 
-    def __enter__(self, *args, **kwargs):
+    def __enter__(self, *args, **kwargs):  # noqa: ARG002
         self.session = requests.Session()
         self.session.mount(
             "https://",
@@ -490,7 +501,7 @@ class LokiLogProvider(LogProvider):
     def close(self) -> None:
         self.session.close()
 
-    def __exit__(self, *args, **kwargs):
+    def __exit__(self, *args, **kwargs):  # noqa: ARG002
         self.close()
 
     def query(
@@ -524,16 +535,17 @@ class LokiLogProvider(LogProvider):
 
 class JobController(BaseController):
     def __init__(
-        self,
-        log: LogsLogger = LogsLogger(),  # The default logger is shared across instances of this class.
-        initialize_auth_api_client: bool = True,
+        self, log: Optional[LogsLogger] = None, initialize_auth_api_client: bool = True
     ):
+        if log is None:
+            log = LogsLogger()
+
         super().__init__(initialize_auth_api_client=initialize_auth_api_client)
         self.log = log
         self.log.open_block("Output")
         self.logs_controller = LogsController()
 
-    def submit(
+    def submit(  # noqa: PLR0913
         self,
         job_config_file: str,
         name: Optional[str] = None,
@@ -548,7 +560,7 @@ class JobController(BaseController):
             config = self.generate_config_from_entrypoint(
                 entrypoint, name, description, workspace_id
             )
-            id = self.submit_from_config(config)
+            id = self.submit_from_config(config)  # noqa: A001
         elif len(entrypoint) == 0:
             # Assume that job_config_file is a file and submit it.
             config = self.generate_config_from_file(
@@ -557,7 +569,7 @@ class JobController(BaseController):
                 description=description,
                 workspace_id=workspace_id,
             )
-            id = self.submit_from_config(config)
+            id = self.submit_from_config(config)  # noqa: A001
         elif len(entrypoint) != 0:
             msg = (
                 "Within an Anyscale Workspace, `anyscale job submit` takes either a file, or a command. To submit a command, use `anyscale job submit -- my command`."
@@ -627,7 +639,7 @@ class JobController(BaseController):
 
         job = self.api_client.create_job_api_v2_decorated_ha_jobs_create_post(
             CreateInternalProductionJob(
-                name=job_config.name or "cli-job-{}".format(datetime.now().isoformat()),
+                name=job_config.name or f"cli-job-{datetime.now().isoformat()}",
                 description=job_config.description or "Job submitted from CLI",
                 project_id=project_id,
                 workspace_id=job_config.workspace_id,
@@ -662,16 +674,17 @@ class JobController(BaseController):
         if not os.path.exists(job_config_file):
             raise click.ClickException(f"Config file {job_config_file} not found.")
 
-        with open(job_config_file, "r") as f:
+        with open(job_config_file) as f:
             config_dict = yaml.safe_load(f)
         return config_dict
 
-    def wait(
+    def wait(  # noqa: PLR0913
         self,
         job_name: Optional[str] = None,
         job_id: Optional[str] = None,
         is_service: bool = False,
         target_state: str = HaJobStates.SUCCESS,
+        interval_secs: float = 10.0,
         timeout_secs=None,
     ):
         if target_state not in HaJobStates.allowable_values:
@@ -686,7 +699,8 @@ class JobController(BaseController):
             ).id,
         )
         with self.log.spinner(f"Waiting for Job `{job_id_or_name}`...") as spinner:
-            for _ in poll(timeout_secs=timeout_secs):
+            state = None
+            for _ in poll(interval_secs=interval_secs, timeout_secs=timeout_secs):
                 state = self._get_job_state(job_id)
                 spinner.text = f"Job `{job_id_or_name}` is in state `{state}`. Waiting to reach state `{target_state}`."
 
@@ -711,7 +725,7 @@ class JobController(BaseController):
         ).result
         return job.state.current_state
 
-    def list(
+    def list(  # noqa: PLR0913
         self,
         include_all_users: bool,
         name: Optional[str],
@@ -960,41 +974,30 @@ class JobController(BaseController):
                 else:
                     return
 
-    def stream_logs(
-        self, job_id: Optional[str] = None, job_name: Optional[str] = None,
-    ) -> None:
-        job_id = cast(str, self._resolve_job_object(job_id, job_name).id)
+    def stream_logs(self, job_id: str) -> None:
         job_run_id = self._wait_for_a_job_run(job_id)
-        websocket_url: str = self.api_client.get_job_logs_stream_api_v2_logs_job_logs_stream_job_id_get(
-            job_id=job_run_id
-        ).result.stream_url
+        websocket_url: str = (
+            self.api_client.get_job_logs_stream_api_v2_logs_job_logs_stream_job_id_get(
+                job_id=job_run_id
+            ).result.stream_url
+        )
 
-        # Implementation is same as
-        # https://github.com/ray-project/ray/blob/6142f52ada995b7123ea7347a70613ca7b924e0c/dashboard/modules/job/sdk.py#L426
-        # However Ray is only part of [all] install, so we replicate the functionality here.
-        async def read_websocket() -> None:
-            async with ClientSession().ws_connect(websocket_url) as ws:
-                while True:
-                    msg = await ws.receive()
-                    if msg.type == WSMsgType.TEXT:
-                        self.log.log(msg.data.strip())
-                    elif msg.type == WSMsgType.CLOSED:
-                        break
-                    elif msg.type == WSMsgType.ERROR:
-                        pass
+        with contextlib.suppress(KeyboardInterrupt):
+            # run_until_complete() is used here b/c the KeyboardInterrupt exception was
+            # not propogated all the way up (ie. it was being handled by internals of asyncio.run())
+            asyncio.get_event_loop().run_until_complete(
+                self._read_websocket(websocket_url, job_run_id)
+            )
 
-        asyncio.run(read_websocket())
-
-    def download_logs(
-        self, job_id: Optional[str] = None, job_name: Optional[str] = None,
-    ) -> None:
-        job_id = cast(str, self._resolve_job_object(job_id, job_name).id)
+    def download_logs(self, job_id: str) -> None:
         prod_job = self._get_latest_job_run(job_id)
         if prod_job.last_job_run is not None:
             cluster_id_of_latest_job_run = prod_job.last_job_run.cluster_id
         else:
             cluster_id_of_latest_job_run = prod_job.state.cluster_id
-        filter = LogFilter(cluster_id=cluster_id_of_latest_job_run, glob="/job-driver-")
+        filter = LogFilter(  # noqa: A001
+            cluster_id=cluster_id_of_latest_job_run, glob="/job-driver-"
+        )
         self.logs_controller.download_logs(filter=filter, write_to_stdout=True)
 
     def logs(
@@ -1002,9 +1005,28 @@ class JobController(BaseController):
         job_id: Optional[str] = None,
         job_name: Optional[str] = None,
         should_follow: bool = False,
-        log_provider_cls: Type[LogProvider] = LokiLogProvider,
     ) -> None:
         job_id = cast(str, self._resolve_job_object(job_id, job_name).id)
+        if should_follow:
+            try:
+                self.stream_logs(job_id)
+                return
+            except Exception as e:  # noqa: BLE001
+                log.info(f"Job logs could not be followed due to error: {e}")
+        try:
+            log.info("Now downloading logs from S3...")
+            self.download_logs(job_id)
+        except Exception as e:  # noqa: BLE001
+            log.info(f"Job logs could not be downloaded due to error: {e}")
+            log.info("Now trying default method of fetching logs.")
+            self.loki_logs(job_id, should_follow)
+
+    def loki_logs(
+        self,
+        job_id: str,
+        should_follow: bool = False,
+        log_provider_cls: Type[LogProvider] = LokiLogProvider,
+    ) -> None:
         self._wait_for_a_job_run(job_id)
 
         start_timestamp_ns = 0
@@ -1081,6 +1103,30 @@ class JobController(BaseController):
                 if jr.job_state in pending_states:
                     self._wait_for_job_running(job_id)
 
+    # Implementation is same as
+    # https://github.com/ray-project/ray/blob/6142f52ada995b7123ea7347a70613ca7b924e0c/dashboard/modules/job/sdk.py#L426
+    # However Ray is only part of [all] install, so we replicate the functionality here.
+    async def _read_websocket(self, websocket_url: str, job_run_id: str) -> None:
+        try:
+            async with ClientSession() as session:
+                async with session.ws_connect(websocket_url) as ws:
+                    while True:
+                        msg = await ws.receive()
+                        if msg.type == WSMsgType.TEXT:
+                            self.log.log(msg.data.strip())
+                        elif msg.type == WSMsgType.CLOSED:
+                            break
+                        elif msg.type == WSMsgType.ERROR:
+                            pass
+        except ClientResponseError as e:
+            # Will return 404 if job run (Ray job) is not found
+            # https://github.com/ray-project/ray/blob/6334576223dd916469fcdac185752b80e0c3e416/dashboard/modules/job/job_head.py#L485
+            if e.status == HTTPNotFound.status_code:
+                raise Exception(
+                    f"Job run {job_run_id} not found in cluster for current running job."
+                )
+            raise e
+
     def retrieve_output(
         self, job_name: Optional[str] = None, job_id: Optional[str] = None,
     ) -> None:
@@ -1093,12 +1139,10 @@ class JobController(BaseController):
                 ha_job_id=job_id
             )
             self.log.log(json.dumps(output.result.output, indent=2))
-        except Exception as e:
+        except Exception as e:  # noqa: BLE001
             traceback.print_exc()
-            try:
+            # Swallow if pretty print logic fails
+            with contextlib.suppress(Exception):
                 j = json.loads(e.body)  # type: ignore
                 detail = j.get("error", {}).get("detail")
                 self.log.error(detail)
-            except Exception:
-                # Swallow if pretty print logic fails
-                pass
