@@ -26,8 +26,11 @@ from anyscale.utils.imports.all import try_import_ray
 
 class ListController(BaseController):
     def __init__(
-        self, log: BlockLogger = BlockLogger(), initialize_auth_api_client: bool = True
+        self, log: Optional[BlockLogger] = None, initialize_auth_api_client: bool = True
     ):
+        if log is None:
+            log = BlockLogger()
+
         super().__init__(initialize_auth_api_client=initialize_auth_api_client)
         self.log = log
         self.log.open_block("Output")
@@ -81,7 +84,7 @@ class ListController(BaseController):
             return f"Projects:\n{table}"
 
     # TODO (jbai): This requires additional refactoring to decouple network fetching from formatting logic
-    def list_sessions(
+    def list_sessions(  # noqa: PLR0912
         self, name: Optional[str], show_all: bool, json_format: bool
     ) -> str:
         project_definition = load_project_or_throw()
@@ -188,10 +191,7 @@ class ListController(BaseController):
                 session_output_strings.append(
                     tabulate.tabulate(
                         snapshot_table,
-                        headers=[
-                            "SNAPSHOT applied to {}".format(session.name),
-                            "APPLIED",
-                        ],
+                        headers=[f"SNAPSHOT applied to {session.name}", "APPLIED",],
                         tablefmt="plain",
                     )
                 )
@@ -204,7 +204,7 @@ class ListController(BaseController):
                             " ".join(
                                 [command.name]
                                 + [
-                                    "{}={}".format(key, val)
+                                    f"{key}={val}"
                                     for key, val in command.params.items()
                                 ]
                             ),
@@ -216,7 +216,7 @@ class ListController(BaseController):
                         tabulate.tabulate(
                             command_table,
                             headers=[
-                                "COMMAND run in {}".format(session.name),
+                                f"COMMAND run in {session.name}",
                                 "ID",
                                 "CREATED",
                             ],
