@@ -1,10 +1,12 @@
 import asyncio
 from datetime import timedelta
+import errno
 import os
 import tempfile
 from typing import List, Optional
 
 import aiohttp
+import click
 from rich.console import Console
 from rich.progress import track
 
@@ -17,6 +19,8 @@ from anyscale.client.openapi_client.models import (
     LogFilter,
 )
 from anyscale.controllers.base_controller import BaseController
+from anyscale.sdk.anyscale_client.models.cluster import Cluster
+from anyscale.sdk.anyscale_client.models.session_state import SessionState
 from anyscale.utils.logs_utils import LogGroup
 
 
@@ -35,8 +39,11 @@ DEFAULT_PARALLELISM = 10
 
 class LogsController(BaseController):
     def __init__(
-        self, log: BlockLogger = BlockLogger(), initialize_auth_api_client: bool = True
+        self, log: Optional[BlockLogger] = None, initialize_auth_api_client: bool = True
     ):
+        if log is None:
+            log = BlockLogger()
+
         super().__init__(initialize_auth_api_client=initialize_auth_api_client)
         self.log = log
         self.console = Console()
@@ -53,26 +60,35 @@ class LogsController(BaseController):
             show_progress_bar=False,
         )
 
-    def get_logs_for_tail(
+    def get_log_group(
         self,
-        filter: LogFilter,
+        filter: LogFilter,  # noqa: A002
         page_size: Optional[int],
         ttl_seconds: Optional[int],
         timeout: timedelta,
-    ):
+    ) -> LogGroup:
+        if filter.cluster_id:
+            cluster: Cluster = self.anyscale_api_client.get_cluster(
+                filter.cluster_id
+            ).result
+            if cluster.state == SessionState.RUNNING:
+                self.log.warning(
+                    "The latest 24 hours of logs are not guaranteed if the cluster is still running. "
+                    "If so, please view logs via the Anyscale UI or Ray dashboard."
+                )
         chunks = self._list_log_chunks(
             log_filter=filter,
             page_size=page_size,
             ttl_seconds=ttl_seconds,
             timeout=timeout,
         )
-        groups = self._group_log_chunk_list(chunks=chunks)
-        return groups
+        log_group = self._group_log_chunk_list(chunks)
+        return log_group
 
-    def download_logs(
+    def download_logs(  # noqa: PLR0913
         self,
         # Provide filters
-        filter: LogFilter,
+        filter: LogFilter,  # noqa: A002
         # List files config
         page_size: int = DEFAULT_PAGE_SIZE,
         ttl_seconds: int = DEFAULT_TTL,
@@ -82,14 +98,8 @@ class LogsController(BaseController):
         parallelism: int = DEFAULT_PARALLELISM,
         download_dir: Optional[str] = None,
         write_to_stdout: bool = False,
-    ):
-        log_chunks: List[LogFileChunk] = self._list_log_chunks(
-            log_filter=filter,
-            page_size=page_size,
-            ttl_seconds=ttl_seconds,
-            timeout=timeout,
-        )
-        log_group: LogGroup = self._group_log_chunk_list(log_chunks)
+    ) -> None:
+        log_group = self.get_log_group(filter, page_size, ttl_seconds, timeout)
         self.console.log(
             f"Discovered {len(log_group.get_files())} log files across {len(log_group.get_chunks())} chunks."
         )
@@ -104,7 +114,7 @@ class LogsController(BaseController):
         )
 
     # TODO (shomilj): Refactor this method. It's too nested.
-    def _download_or_stdout(
+    def _download_or_stdout(  # noqa: PLR0913
         self,
         log_group: LogGroup,
         parallelism: int,
@@ -117,78 +127,87 @@ class LogsController(BaseController):
         if len(log_group.get_chunks()) == 0:
             return
 
-        with tempfile.TemporaryDirectory() as tmp_dir:
-            # Download all files to a temporary directory.
-            asyncio.run(
-                # TODO (shomilj): Add efficient tailing method here.
-                self._download_files(
-                    base_folder=tmp_dir,
-                    log_chunks=log_group.get_chunks(),
-                    parallelism=parallelism,
-                    read_timeout=read_timeout,
-                    show_progress_bar=show_progress_bar,
-                )
-            )
-
-            for log_file in log_group.get_files():
-                is_tail = tail > 0
-                chunks = log_file.get_chunks(reverse=is_tail)
-                if write_to_stdout:
-                    # Write to standard out
-                    lines_read = 0
-                    for chunk in chunks:
-                        with open(
-                            os.path.join(tmp_dir, chunk.chunk_name), "r"
-                        ) as source:
-                            if tail > 0:
-                                # Tail is enabled, so read log lines in reverse.
-                                # TODO (shomilj): Make this more efficient (don't read everything into memory before reversing it)
-                                # For now, this is fine (the chunks are <10 MB, so this isn't that inefficient).
-                                for line in reversed(source.readlines()):
-                                    print(line.strip())
-                                    lines_read += 1
-                                    if lines_read >= tail:
-                                        return
-                            else:
-                                # Read log lines normally.
-                                print(source.read())
-                else:
-                    # Write to destination files
-                    real_path = os.path.join(
-                        download_dir or "", log_file.get_target_path()
+        try:
+            with tempfile.TemporaryDirectory() as tmp_dir:
+                # Download all files to a temporary directory.
+                asyncio.run(
+                    # TODO (shomilj): Add efficient tailing method here.
+                    self._download_files(
+                        base_folder=tmp_dir,
+                        log_chunks=log_group.get_chunks(),
+                        parallelism=parallelism,
+                        read_timeout=read_timeout,
+                        show_progress_bar=show_progress_bar,
                     )
-                    real_dir = os.path.dirname(real_path)
-                    if not os.path.exists(real_dir):
-                        os.makedirs(real_dir)
+                )
 
-                    chunks_written = 0
-                    with open(real_path, "w") as dest:
+                for log_file in log_group.get_files():
+                    is_tail = tail > 0
+                    chunks = log_file.get_chunks(reverse=is_tail)
+                    if write_to_stdout:
+                        # Write to standard out
+                        lines_read = 0
                         for chunk in chunks:
-                            downloaded_chunk_path = os.path.join(
-                                tmp_dir, chunk.chunk_name
-                            )
-                            if not os.path.exists(downloaded_chunk_path):
-                                self.log.error(
-                                    "Download failed for file: ", chunk.chunk_name
+                            with open(
+                                os.path.join(tmp_dir, chunk.chunk_name)
+                            ) as source:
+                                if tail > 0:
+                                    # Tail is enabled, so read log lines in reverse.
+                                    # TODO (shomilj): Make this more efficient (don't read everything into memory before reversing it)
+                                    # For now, this is fine (the chunks are <10 MB, so this isn't that inefficient).
+                                    for line in reversed(source.readlines()):
+                                        print(line.strip())
+                                        lines_read += 1
+                                        if lines_read >= tail:
+                                            return
+                                else:
+                                    # Read log lines normally.
+                                    print(source.read())
+                    else:
+                        # Write to destination files
+                        real_path = os.path.join(
+                            download_dir or "", log_file.get_target_path()
+                        )
+                        real_dir = os.path.dirname(real_path)
+                        if not os.path.exists(real_dir):
+                            os.makedirs(real_dir)
+
+                        chunks_written = 0
+                        with open(real_path, "w") as dest:
+                            for chunk in chunks:
+                                downloaded_chunk_path = os.path.join(
+                                    tmp_dir, chunk.chunk_name
                                 )
-                                continue
-                            with open(downloaded_chunk_path, "r") as source:
-                                for line in source:
-                                    dest.write(line)
-                                dest.write("\n")
-                            chunks_written += 1
+                                if not os.path.exists(downloaded_chunk_path):
+                                    self.log.error(
+                                        "Download failed for file: ", chunk.chunk_name
+                                    )
+                                    continue
+                                with open(
+                                    downloaded_chunk_path, errors="ignore"
+                                ) as source:
+                                    for line in source:
+                                        dest.write(line)
+                                    dest.write("\n")
+                                chunks_written += 1
 
-                    if chunks_written == 0:
-                        os.remove(real_path)
+                        if chunks_written == 0:
+                            os.remove(real_path)
 
-            if not write_to_stdout:
-                if not download_dir:
-                    download_dir = os.getcwd()
-                sample_chunk = log_group.get_chunks()[0]
-                cluster_id = sample_chunk.cluster_id
-                download_dir = download_dir + f"/logs/{cluster_id}"
-                self.console.log(
-                    f"Download complete! Files have been downloaded to {download_dir}"
+                if not write_to_stdout:
+                    if not download_dir:
+                        download_dir = os.getcwd()
+                    sample_chunk = log_group.get_chunks()[0]
+                    cluster_id = sample_chunk.cluster_id
+                    download_dir = download_dir + f"/logs/{cluster_id}"
+                    self.console.log(
+                        f"Download complete! Files have been downloaded to {download_dir}"
+                    )
+        except OSError as err:
+            if err.errno == errno.EMFILE:
+                raise click.ClickException(
+                    "Too many open files. Try doubling your open files limit by running \n"
+                    "ulimit -n $(($(ulimit -n) * 2))"
                 )
 
     def _group_log_chunk_list(self, chunks: List[LogFileChunk]) -> LogGroup:
@@ -236,13 +255,13 @@ class LogsController(BaseController):
 
         return all_log_chunks
 
-    async def _download_file(
+    async def _download_file(  # noqa: PLR0913
         self,
         sem: asyncio.Semaphore,
-        pos: int,
+        pos: int,  # noqa: ARG002
         file_name: str,
         url: str,
-        size: int,
+        size: int,  # noqa: ARG002
         session: aiohttp.ClientSession,
         read_timeout: timedelta,
     ) -> None:
@@ -264,7 +283,7 @@ class LogsController(BaseController):
                         f"Unable to download file {file_name}! response: [{response.status}, {await response.text()}]"
                     )
 
-    async def _download_files(
+    async def _download_files(  # noqa: PLR0913
         self,
         base_folder: Optional[str],
         log_chunks: List[LogFileChunk],
