@@ -18,6 +18,10 @@ from anyscale.util import get_endpoint, is_anyscale_workspace
 from anyscale.utils.runtime_env import override_runtime_env_for_local_working_dir
 
 
+def _is_v2_service_id(service_id: str) -> bool:
+    return service_id.startswith("service2")
+
+
 class ServiceController(BaseController):
     def __init__(
         self, log: Optional[BlockLogger] = None, initialize_auth_api_client: bool = True
@@ -31,6 +35,78 @@ class ServiceController(BaseController):
             initialize_auth_api_client=initialize_auth_api_client
         )
 
+    def _get_service_id_from_name(
+        self, service_name: str, project_id: Optional[str]
+    ) -> str:
+        """Get the ID for a service by name.
+
+        If project_id is specified, filter to that project, else use the default infer_project_id logic.
+
+        Raises an exception if there are zero or multiple services with the given name.
+        """
+        if project_id is None:
+            project_id = infer_project_id(
+                project_id, self.anyscale_api_client, self.log
+            )
+
+        results = self.api_client.list_services_api_v2_services_v2_get(
+            project_id=project_id, name=service_name,
+        ).results
+
+        if len(results) == 0:
+            raise click.ClickException(
+                f"No service with name '{service_name}' was found. "
+                "Please verify that this service exists and you have access to it."
+            )
+        elif len(results) > 1:
+            raise click.ClickException(
+                f"There are multiple services with name '{service_name}'. "
+                "Please specify the --service-id instead."
+            )
+
+        return results[0].id
+
+    def get_service_id(
+        self,
+        service_id: Optional[str],
+        service_name: Optional[str],
+        service_config_file: Optional[str],
+    ) -> str:
+        """Get the service ID given the ID, name, or config file.
+
+        This is a utility used by multiple CLI commands to standardize mapping these options
+        to a service_id.
+
+        The precedence is: service_id > service_name > service_config_file.
+
+        If the service_config_file is provided and it has a project_id, that will be used to
+        filter the query. Else this uses the infer_project_id default logic.
+        """
+        if service_id is not None:
+            if service_name is not None or service_config_file is not None:
+                raise click.ClickException(
+                    "Only one of service ID, name, or config file should be specified."
+                )
+        elif service_name is not None:
+            if service_config_file is not None:
+                raise click.ClickException(
+                    "Only one of service ID, name, or config file should be specified."
+                )
+            service_id = self._get_service_id_from_name(service_name, None)
+        elif service_config_file is not None:
+            service_config: ServiceConfig = self._read_service_config_file(
+                service_config_file
+            )
+            service_id = self._get_service_id_from_name(
+                service_config.name, service_config.project_id
+            )
+        else:
+            raise click.ClickException(
+                "Service ID, name, or config file must be specified."
+            )
+
+        return service_id
+
     def deploy(  # noqa: PLR0913
         self,
         service_config_file: str,
@@ -56,7 +132,7 @@ class ServiceController(BaseController):
             config.project_id = project_id
             self.deploy_from_config(config)
         elif len(entrypoint) == 0:
-            # Assume that job_config_file is a file and submit it.
+            # Assume that service_config_file is a file and submit it.
             config = self.generate_config_from_file(
                 service_config_file, name, description, healthcheck_url=healthcheck_url,
             )
@@ -101,13 +177,9 @@ class ServiceController(BaseController):
         version: Optional[str] = None,
         canary_percent: Optional[int] = None,
     ) -> ServiceConfig:
-        if not os.path.exists(service_config_file):
-            raise click.ClickException(f"Config file {service_config_file} not found.")
-
-        with open(service_config_file) as f:
-            config_dict = yaml.safe_load(f)
-
-        service_config = self._populate_service_config(config_dict)
+        service_config: ServiceConfig = self._read_service_config_file(
+            service_config_file
+        )
         if name:
             service_config.name = name
 
@@ -195,6 +267,15 @@ class ServiceController(BaseController):
         service_config = ServiceConfig.parse_obj(config_dict)
         return service_config
 
+    def _read_service_config_file(self, service_config_file: str) -> ServiceConfig:
+        if not os.path.exists(service_config_file):
+            raise click.ClickException(f"Config file {service_config_file} not found.")
+
+        with open(service_config_file) as f:
+            config_dict = yaml.safe_load(f)
+
+        return self._populate_service_config(config_dict)
+
     def rollout(
         self,
         service_config_file: str,
@@ -202,6 +283,9 @@ class ServiceController(BaseController):
         version: Optional[str] = None,
         canary_percent: Optional[int] = None,
     ):
+        """
+        Deploys a Service 2.0.
+        """
         config = self.generate_config_from_file(
             service_config_file,
             name=name,
@@ -242,18 +326,35 @@ class ServiceController(BaseController):
             is_service=True,
         )
 
-    def archive(self, service_id: Optional[str], service_name: Optional[str]) -> None:
-        self.job_controller.archive(service_id, service_name, is_service=True)
+    def archive(self, service_id: str):
+        if _is_v2_service_id(service_id):
+            raise click.ClickException(
+                "archive is not currently supported for v2 services."
+                "Please contact Anyscale support for more information."
+            )
+
+        self.job_controller.archive(service_id, None, is_service=True)
 
     def rollback(self, service_id: str) -> None:
+        if not _is_v2_service_id(service_id):
+            raise click.ClickException(
+                "rollback is only supported for v2 services."
+                "Please contact Anyscale support for more information."
+            )
+
         service = self.api_client.rollback_service_api_v2_services_v2_service_id_rollback_post(
             service_id
         ).result
 
-        self.log.info(f"Service {service.id} has been rolled back")
+        self.log.info(f"Service {service.id} rollback initiated.")
         self.log.info(
             f'View the service in the UI at {get_endpoint(f"/services/{service.id}")}.'
         )
 
-    def terminate(self, service_id: Optional[str], service_name: Optional[str]) -> None:
-        self.job_controller.terminate(service_id, service_name, is_service=True)
+    def terminate(self, service_id: str) -> None:
+        if _is_v2_service_id(service_id):
+            self.api_client.terminate_service_api_v2_services_v2_service_id_terminate_post(
+                service_id
+            )
+        else:
+            self.job_controller.terminate(service_id, None, is_service=True)
