@@ -1,7 +1,7 @@
 import os
 import shlex
 import subprocess
-from typing import Any, Dict, List, Sequence
+from typing import Any, Dict, List, Optional, Sequence
 
 from pydantic import BaseModel
 
@@ -11,7 +11,8 @@ from anyscale.client.openapi_client.models.experimental_workspace import (
 )
 from anyscale.cluster_config import configure_for_session
 from anyscale.controllers.base_controller import BaseController
-from anyscale.shared_anyscale_utils.util import get_container_name
+from anyscale.sdk.anyscale_client.models.cloud_version import CloudVersion
+from anyscale.shared_anyscale_utils.conf import V2_RAY_MOUNT_PATH
 from anyscale.util import get_working_dir
 from anyscale.utils.imports.all import try_import_ray
 from anyscale.workspace import load_workspace_id_or_throw, write_workspace_id_to_disk
@@ -21,12 +22,16 @@ class WorkspaceCommandContext(BaseModel):
     cluster_config: Dict[str, Any]
     head_ip: str
     project_id: str
+    cluster_id: str
 
 
 class WorkspaceController(BaseController):
     def __init__(
-        self, log: BlockLogger = BlockLogger(), initialize_auth_api_client: bool = True
+        self, log: Optional[BlockLogger] = None, initialize_auth_api_client: bool = True
     ):
+        if log is None:
+            log = BlockLogger()
+
         super().__init__(initialize_auth_api_client=initialize_auth_api_client)
         self.log = log
 
@@ -68,25 +73,31 @@ class WorkspaceController(BaseController):
         ).result.head_ip
 
         return WorkspaceCommandContext(
-            cluster_config=cluster_config, head_ip=head_ip, project_id=project_id
+            cluster_config=cluster_config,
+            head_ip=head_ip,
+            project_id=project_id,
+            cluster_id=cluster_id,
         )
 
     def run_cmd(self, ssh_option: Sequence[str], cmd: str):
         workspace_command_context = self._get_workspace_command_context()
         cluster_config = workspace_command_context.cluster_config
-        head_ip = workspace_command_context.head_ip
 
-        ssh_user = cluster_config["auth"]["ssh_user"]
         key_path = cluster_config["auth"]["ssh_private_key"]
-        container_name = get_container_name(cluster_config)
+
+        head_ip = workspace_command_context.head_ip
+        ssh_user = cluster_config["auth"]["ssh_user"]
+        target_host = f"{ssh_user}@{head_ip}"
 
         command = shlex.quote(cmd)
+
         ssh_command = (
             ["ssh"]
             + list(ssh_option)
+            + ["-o", f"ProxyCommand=ssh -W %h:%p -i {key_path} {target_host}"]
             + ["-tt", "-i", key_path]
-            + ["{}@{}".format(ssh_user, head_ip)]
-            + [f"docker exec -it {container_name} sh -c {command}"]
+            + ["ray@0.0.0.0", "-p", "5020"]
+            + [f"sh -c {command}"]
         )
 
         subprocess.run(ssh_command)  # noqa: B1
@@ -107,6 +118,7 @@ class WorkspaceController(BaseController):
         cluster_config = workspace_command_context.cluster_config
         head_ip = workspace_command_context.head_ip
         project_id = workspace_command_context.project_id
+        cluster_id = workspace_command_context.cluster_id
 
         ssh_user = cluster_config["auth"]["ssh_user"]
         key_path = cluster_config["auth"]["ssh_private_key"]
@@ -115,11 +127,14 @@ class WorkspaceController(BaseController):
 
         directory_name = get_working_dir(cluster_config, project_id, self.api_client)
 
-        source_directory = (
-            get_docker_host_mount_location(cluster_config["cluster_name"])
-            + directory_name
-            + "/"
+        cloud_id = self.anyscale_api_client.get_session(cluster_id).result.cloud_id
+        cloud_version = self.anyscale_api_client.get_cloud(cloud_id).result.version
+        mount_path = (
+            V2_RAY_MOUNT_PATH
+            if not cloud_version or cloud_version == CloudVersion.V2
+            else get_docker_host_mount_location(cluster_config["cluster_name"])
         )
+        source_directory = mount_path + directory_name + "/"
 
         rsync_command = [
             "rsync",
@@ -133,17 +148,17 @@ class WorkspaceController(BaseController):
             rsync_command.extend(["--exclude", rsync_exclude])
 
         for rsync_filter in rsync_filters:
-            rsync_command.extend(["--filter", "dir-merge,- {}".format(rsync_filter)])
+            rsync_command.extend(["--filter", f"dir-merge,- {rsync_filter}"])
 
         if down:
             rsync_command += [
-                "{}@{}:{}".format(ssh_user, head_ip, source_directory),
+                f"{ssh_user}@{head_ip}:{source_directory}",
                 local_path,
             ]
         else:
             rsync_command += [
                 local_path,
-                "{}@{}:{}".format(ssh_user, head_ip, source_directory),
+                f"{ssh_user}@{head_ip}:{source_directory}",
             ]
 
         subprocess.run(rsync_command)  # noqa: B1
