@@ -3,8 +3,10 @@ import datetime
 import json
 import logging
 import os
+import re
 import shutil
 import subprocess
+import tarfile
 import tempfile
 import time
 from typing import Any, Dict, Optional, Tuple
@@ -54,6 +56,13 @@ WORKSPACE_ID = os.environ.get("ANYSCALE_EXPERIMENTAL_WORKSPACE_ID", "")
 USERNAME = os.environ.get("ANYSCALE_EXPERIMENTAL_USERNAME", "unknown_user")
 BASE_SNAPSHOT = os.environ.get("ANYSCALE_EXPERIMENTAL_BASE_SNAPSHOT") or None
 
+# GitHub URI to use as the template for the new workspace. This is the URI you see
+# when browsing the desired subdir in the web UI, e.g.:
+#
+# https://github.com/ray-project/ray/tree/master/release/ml_user_tests
+#
+WORKSPACE_TEMPLATE = os.environ.get("ANYSCALE_WORKSPACE_TEMPLATE") or None
+
 # Other debug configs.
 SNAPSHOT_INTERVAL = int(os.environ.get("WORKSPACE_SNAPSHOT_INTERVAL", 300))
 EFS_WORKSPACE_DIR = os.environ.get("EFS_WORKSPACE_DIR", "/efs/workspaces")
@@ -186,6 +195,68 @@ def unpack_snapshot_zip(zip_path: str, directory: str) -> None:
     )
 
 
+def unpack_github_dir(github_tree_uri: str, out_dir: str) -> None:
+    """Download a GitHub tree URI for a directory.
+
+    Args:
+        github_tree_uri: URL for a directory, for example:
+            "https://github.com/ray-project/ray/tree/master/release/ml_user_tests"
+        out_dir: The output dir to put files, which must be empty.
+    """
+
+    if "github.com" not in github_tree_uri:
+        raise ValueError("Only GitHub URLs are supported by load_package().")
+
+    # Normalize dir URIs.
+    if github_tree_uri.endswith("/"):
+        github_tree_uri = github_tree_uri[:-1]
+
+    URL_FORMAT = ".*github.com/([^/]*)/([^/]*)/tree/([^/]*)/(.*)"
+    match = re.match(URL_FORMAT, github_tree_uri)
+    if not match:
+        raise ValueError(f"GitHub URL must be of format {URL_FORMAT}")
+    gh_user = match.group(1)
+    gh_repo = match.group(2)
+    gh_branch = match.group(3)
+    gh_subdir = match.group(4)
+
+    logger.info(
+        f"Attempting download, gh_user={gh_user}, gh_repo={gh_repo}, "
+        f"gh_branch={gh_branch}, gh_subdir={gh_subdir}"
+    )
+
+    tmp = tempfile.mktemp(prefix=f"github_{gh_repo}", suffix=".tar.gz")
+    subprocess.check_call(
+        [
+            "curl",
+            "--fail",
+            "-L",
+            f"https://github.com/{gh_user}/{gh_repo}/tarball/{gh_branch}",
+            "--output",
+            tmp,
+        ]
+    )
+    logger.info(f"Downloaded tarball to {tmp}")
+
+    # The directory we want to extract is a subdir within the zip, e.g.,
+    # ray-project-fc17342/release/ml_user_tests/ray-lightning
+    top_dir = tarfile.open(tmp).getnames()[0]
+    tar_path = top_dir + "/" + gh_subdir
+    # Calculate the depth to the subdir and strip those leading components.
+    strip = tar_path.count("/") + 1
+    tar_command = [
+        "tar",
+        "xzf",
+        tmp,
+        "-C",
+        out_dir,
+        tar_path,
+        f"--strip-components={strip}",
+    ]
+    logger.info(f"Extract command is {tar_command}")
+    subprocess.check_call(tar_command)
+
+
 def compute_content_hash(zip_path: str) -> bytes:
     """Return the md5 hash of a given zipfile on disk."""
     md5 = subprocess.check_output(  # noqa
@@ -391,6 +462,9 @@ def restore_latest():
     latest = find_latest(ignore_base_snapshot=False)
     logger.info(f"Latest snapshot found was {latest}")
     if not latest:
+        # No snapshot found: fallback to template or blank workspace.
+        if WORKSPACE_TEMPLATE:
+            unpack_github_dir(WORKSPACE_TEMPLATE, WORKING_DIR)
         return
     workspace_dir = os.path.join(EFS_WORKSPACE_DIR, WORKSPACE_ID)
     unpack_snapshot_zip(latest, WORKING_DIR)
