@@ -6,9 +6,7 @@ import logging
 import os
 import random
 import string
-import subprocess
 import sys
-import threading
 import time
 from typing import (
     Any,
@@ -16,16 +14,18 @@ from typing import (
     cast,
     Dict,
     Generator,
-    Iterator,
     List,
     Optional,
+    Set,
     Tuple,
+    TypeVar,
     Union,
 )
 from urllib.parse import quote as quote_sanitize, urlencode, urljoin
 import webbrowser
 
 import boto3
+from boto3.resources.base import ServiceResource as Boto3Resource
 from botocore.config import Config
 from botocore.exceptions import ClientError, NoRegionError
 import click
@@ -55,7 +55,7 @@ logger = logging.getLogger(__file__)
 BOTO_MAX_RETRIES = 5
 PROJECT_NAME_ENV_VAR = "ANYSCALE_PROJECT_NAME"
 
-VALID_PYTHON_VERSIONS = ["py36", "py37", "py38", "py39"]
+VALID_BYOD_PYTHON_VERSIONS = ["py36", "py37", "py38", "py39"]
 
 log = BlockLogger()  # Anyscale CLI Logger
 
@@ -64,7 +64,7 @@ VPC_CIDR_RANGE = "10.0.0.0/16"
 DEFAULT_RAY_VERSION = "1.7.0"
 
 
-def confirm(msg: str, yes: bool) -> Any:
+def confirm(msg: str, yes: bool) -> Optional[bool]:
     return None if yes else click.confirm(msg, abort=True)
 
 
@@ -109,11 +109,9 @@ def send_json_request_raw(
         elif method == "PUT":
             resp = requests.put(url, json=json_args, cookies=cookies)
         else:
-            assert False, "unknown method {}".format(method)
+            raise AssertionError(f"unknown method {method}")
     except requests.exceptions.ConnectionError:
-        raise click.ClickException(
-            "Failed to connect to anyscale server at {}".format(url)
-        )
+        raise click.ClickException(f"Failed to connect to anyscale server at {url}")
 
     return resp
 
@@ -137,7 +135,7 @@ def send_json_request(
                 "with the token '{}'.".format(resp.headers["x-trace-id"])
             )
 
-        raise click.ClickException("{}: {}.".format(resp.status_code, resp.text))
+        raise click.ClickException(f"{resp.status_code}: {resp.text}.")
 
     if resp.status_code == 204:
         return {}
@@ -150,7 +148,7 @@ def send_json_request(
 
 
 def deserialize_datetime(s: str) -> datetime.datetime:
-    if sys.version_info < (3, 7) and ":" == s[-3:-2]:
+    if sys.version_info < (3, 7) and s[-3:-2] == ":":
         s = s[:-3] + s[-2:]
 
     return datetime.datetime.strptime(s, "%Y-%m-%dT%H:%M:%S.%f%z")
@@ -177,11 +175,9 @@ def humanize_timestamp(timestamp: datetime.datetime) -> str:
         return "{} second{} ago".format(delta_s, "s" if delta_s > 1 else "")
 
 
-def get_cluster_config(config_path: str) -> Any:
+def get_cluster_config(config_path: str) -> Dict:
     with open(config_path) as f:
-        cluster_config = yaml.safe_load(f)
-
-    return cluster_config
+        return yaml.safe_load(f)
 
 
 def get_requirements(requirements_path: str) -> str:
@@ -189,21 +185,21 @@ def get_requirements(requirements_path: str) -> str:
         return f.read()
 
 
-def _resource(name: str, region: str) -> Any:
+def _resource(name: str, region: str) -> Boto3Resource:
     boto_config = Config(retries={"max_attempts": BOTO_MAX_RETRIES})
-    return boto3.resource(name, region, config=boto_config)
+    return boto3.resource(name, region, config=boto_config)  # type: ignore
 
 
 def _client(name: str, region: str) -> Any:
     return _resource(name, region).meta.client
 
 
-def _get_role(role_name: str, region: str) -> Any:
+def _get_role(role_name: str, region: str) -> Optional[Boto3Resource]:
     iam = _resource("iam", region)
-    role = iam.Role(role_name)
+    role = iam.Role(role_name)  # type: ignore
     try:
         role.load()
-        return role
+        return role  # type: ignore
     except ClientError as exc:
         if exc.response.get("Error", {}).get("Code") == "NoSuchEntity":
             return None
@@ -211,17 +207,30 @@ def _get_role(role_name: str, region: str) -> Any:
             raise exc
 
 
-def _get_subnet(subnet_arn: str, region: str) -> Any:
+def _get_aws_efs_mount_target_ip(boto3_session: Any, efs_id: str) -> Optional[str]:
+    client = boto3_session.client("efs")
+    try:
+        mount_targets_response = client.describe_mount_targets(FileSystemId=efs_id)
+        if not mount_targets_response.get("MountTargets"):
+            logger.warning(f"EFS with id {efs_id} does not contain mount targets.")
+            return None
+        return mount_targets_response.get("MountTargets")[0].get("IpAddress")
+    except ClientError as e:
+        if e.response["Error"]["Code"] == "FileSystemNotFound":
+            return None
+        raise e
+
+
+def _get_subnet(subnet_arn: str, region: str) -> Optional[Boto3Resource]:
     ec2 = _resource("ec2", region)  # TODO: take a resource as an argument
-    subnet = ec2.Subnet(subnet_arn)
+    subnet = ec2.Subnet(subnet_arn)  # type: ignore
     try:
         subnet.load()
-        return subnet
+        return subnet  # type: ignore
     except ClientError as e:
-        if e.response.get("Error", {}).get("Code") == "NoSuchEntity":
+        if e.response["Error"]["Code"] == "InvalidSubnetID.NotFound":
             return None
-        else:
-            raise e
+        raise e
 
 
 def get_available_regions() -> List[str]:
@@ -241,7 +250,7 @@ def get_availability_zones(region: str) -> List[str]:
     ]
 
 
-def launch_gcp_cloud_setup(
+def launch_gcp_cloud_setup(  # noqa: PLR0913
     name: str,
     region: str,
     is_k8s: Optional[bool],
@@ -280,69 +289,6 @@ def launch_gcp_cloud_setup(
     webbrowser.open(full_url)
 
 
-class Timer:
-    """
-    Code adopted from https://stackoverflow.com/a/39504463/3727678
-    Spawn thread and time process that may be blocking.
-    """
-
-    def timer_generator(self) -> Iterator[str]:
-        while True:
-            time_diff = time.gmtime(time.time() - self.start_time)
-            yield "{0}: {1}".format(self.message, time.strftime("%M:%S", time_diff))
-
-    def __init__(self, message: str = "") -> None:
-        self.message = message
-        self.busy = False
-        self.start_time = 0.0
-
-    def timer_task(self) -> None:
-        while self.busy:
-            sys.stdout.write(next(self.timer_generator()))
-            sys.stdout.flush()
-            time.sleep(0.1)
-            sys.stdout.write("\b" * (len(self.message) + 20))
-            sys.stdout.flush()
-
-    def start(self) -> None:
-        self.busy = True
-        self.start_time = time.time()
-        threading.Thread(target=self.timer_task).start()
-
-    def stop(self) -> None:
-        sys.stdout.write("\n")
-        sys.stdout.flush()
-        self.busy = False
-        self.start_time = 0.0
-        time.sleep(1)
-
-
-def check_is_feature_flag_on(flag_key: str, default: bool = False) -> bool:
-    try:
-        is_on = send_json_request(
-            "/api/v2/userinfo/check_is_feature_flag_on", {"flag_key": flag_key},
-        )
-    except Exception:
-        return default
-
-    return cast(bool, is_on["result"]["is_on"])
-
-
-def get_active_sessions(
-    project_id: str, session_name: str, api_client: Optional[ProductApi]
-) -> Any:
-    if api_client:
-        return api_client.list_sessions_api_v2_sessions_get(
-            project_id=project_id, name=session_name, active_only=True
-        ).results
-    else:
-        response = anyscale.util.send_json_request(
-            "/api/v2/sessions/",
-            {"project_id": project_id, "name": session_name, "active_only": True},
-        )
-        return response["results"]
-
-
 def get_project_directory_name(project_id: str, api_client: ProductApi = None) -> str:
     if api_client is None:
         api_client = get_auth_api_client().api_client
@@ -528,7 +474,7 @@ def _get_rsync_args(rsync_exclude: List[str], rsync_filter: List[str]) -> List[s
     ]
 
 
-def get_rsync_command(
+def get_rsync_command(  # noqa: PLR0913
     ssh_command: List[str],
     source: str,
     ssh_user: str,
@@ -568,7 +514,7 @@ def get_rsync_command(
 
     command_list += [
         source,
-        "{}@{}:{}".format(ssh_user, head_ip, target),
+        f"{ssh_user}@{head_ip}:{target}",
     ]
     return command_list, env
 
@@ -580,16 +526,15 @@ def populate_session_args(cluster_config_str: str, config_file_name: str) -> str
     t = env.parse(cluster_config_str)
     for elem in t.body[0].nodes:  # type: ignore
 
-        if isinstance(elem, jinja2.nodes.Getattr):  # type: ignore
-            if elem.attr not in os.environ:
-                prefixed_command = " ".join(
-                    [f"{elem.attr}=<value>", "anyscale"] + sys.argv[1:]
-                )
-                raise click.ClickException(
-                    f"\tThe environment variable {elem.attr} was not set, yet it is required "
-                    f"for configuration file {config_file_name}.\n\tPlease specify {elem.attr} "
-                    f"by prefixing the command.\n\t\t{prefixed_command}"
-                )
+        if isinstance(elem, jinja2.nodes.Getattr) and elem.attr not in os.environ:
+            prefixed_command = " ".join(
+                [f"{elem.attr}=<value>", "anyscale"] + sys.argv[1:]
+            )
+            raise click.ClickException(
+                f"\tThe environment variable {elem.attr} was not set, yet it is required "
+                f"for configuration file {config_file_name}.\n\tPlease specify {elem.attr} "
+                f"by prefixing the command.\n\t\t{prefixed_command}"
+            )
 
     template = jinja2.Template(cluster_config_str)
     cluster_config_filled = template.render(env=os.environ)
@@ -627,42 +572,11 @@ def get_user_info() -> Optional[UserInfo]:
     return api_client.get_user_info_api_v2_userinfo_get().result
 
 
-def get_anyscale_version() -> str:
-    try:
-        # Return git sha if anyscale pip package was built in development mode
-        # TODO (aguo): Pip version 21.2.1 made a change to the FrozenRequirement api. Get rid
-        # of this interal api usage.
-        from pip._internal.operations.freeze import FrozenRequirement
-        import pkg_resources
-
-        distributions = {v.key: v for v in pkg_resources.working_set}
-        distribution = distributions["anyscale"]
-        frozen_requirement = FrozenRequirement.from_dist(distribution)  # type: ignore
-        if frozen_requirement.editable:
-            try:
-                return (
-                    subprocess.check_output(  # noqa: B1
-                        ["git", "describe", "--always"],
-                        cwd=os.path.dirname(os.path.realpath(__file__)),
-                    )  # noqa: B1
-                    .strip()
-                    .decode("utf-8")
-                )
-            except subprocess.CalledProcessError:
-                log.warning(
-                    "Not a git repository, will use standard release versioning"
-                )
-    except Exception:
-        # None of this is critical behavior and so this should not crash the CLI if we can't get a git version.
-        pass
-    return anyscale.__version__
-
-
 def generate_slug(length: int = 6) -> str:
     return "".join(random.choices(string.ascii_lowercase + string.digits, k=length))
 
 
-def validate_non_negative_arg(ctx, param, value):
+def validate_non_negative_arg(ctx, param, value):  # noqa: ARG001
     """
     Checks that an integer option to click command is non-negative.
     """
@@ -701,6 +615,7 @@ def _update_external_ids_for_policy(
 
 def extract_versions_from_image_name(image_name: str) -> Tuple[str, str]:
     """Returns the python version and ray extracted from an image tag.
+    This should be used when creating BYOD images.
 
     Args:
         image_name: e.g. anyscale/ray-ml:1.11.1-py38-gpu
@@ -748,13 +663,16 @@ def _ray_version_major_minor(ray_version: str) -> Tuple[int, int]:
 
 def _check_python_version(python_version: str) -> None:
     assert (
-        python_version in VALID_PYTHON_VERSIONS
-    ), f"Expected python_version to be one of {VALID_PYTHON_VERSIONS}, got {python_version}."
+        python_version in VALID_BYOD_PYTHON_VERSIONS
+    ), f"Expected python_version to be one of {VALID_BYOD_PYTHON_VERSIONS}, got {python_version}."
 
 
 def sleep_till(wake_time: float) -> None:
     """Sleep till the designated time"""
-    time.sleep(max(0, wake_time - time.time()))
+    now = time.time()
+    if now >= wake_time:
+        return
+    time.sleep(wake_time - now)
 
 
 def poll(
@@ -765,18 +683,19 @@ def poll(
     """Poll every interval_secs, until timeout_secs, or max iterations has been reached.
     Yield the iteration number, starting at 1.
     """
-    last_poll_time = time.time()
-    end_time = time.time() + timeout_secs if timeout_secs else None
+    now = time.time()
+    end_time = now + timeout_secs if timeout_secs else None
     count = 0
     should_continue_iter = max_iter is None or count < max_iter
-    should_continue_time = end_time is None or time.time() < end_time
+    should_continue_time = end_time is None or now < end_time
     while should_continue_iter and should_continue_time:
         count += 1
-        last_poll_time = time.time()
+        wake_time = now + interval_secs
         yield count
-        sleep_till(last_poll_time + interval_secs)
+        sleep_till(wake_time)
+        now = time.time()
         should_continue_iter = max_iter is None or count < max_iter
-        should_continue_time = end_time is None or time.time() < end_time
+        should_continue_time = end_time is None or now < end_time
 
 
 def is_anyscale_workspace() -> bool:
@@ -817,19 +736,19 @@ def get_user_env_aws_account(region: str) -> str:
 def prepare_cloudformation_template(
     region: str, cfn_stack_name: str, cloud_id: str
 ) -> str:
-    with open(f"{anyscale.conf.ROOT_DIR_PATH}/anyscale-cloud-setup.yaml", "r") as f:
+    with open(f"{anyscale.conf.ROOT_DIR_PATH}/anyscale-cloud-setup.yaml") as f:
         body = f.read()
 
     azs = get_availability_zones(region)
     subnet_templates: List[str] = []
     subnets_route_table_association: List[str] = []
-    subnets_references: List[str] = []
+    subnets_with_availability_zones: List[str] = []
 
     vpc_cidr = ipaddress.ip_network(VPC_CIDR_RANGE)
     if len(azs) > 4:
-        subnet_cidrs = [s for s in vpc_cidr.subnets(prefixlen_diff=3)]
+        subnet_cidrs = list(vpc_cidr.subnets(prefixlen_diff=3))
     else:
-        subnet_cidrs = [s for s in vpc_cidr.subnets(prefixlen_diff=2)]
+        subnet_cidrs = list(vpc_cidr.subnets(prefixlen_diff=2))
 
     for i, az in enumerate(azs):
         subnet_templates.append(
@@ -857,13 +776,18 @@ def prepare_cloudformation_template(
         SubnetId: !Ref Subnet{i}"""
         )
 
-        subnets_references.append(f"!Ref Subnet{i}")
+        subnets_with_availability_zones.append(
+            f'{{"subnet_id": !Ref Subnet{i}, "availability_zone": !GetAtt Subnet{i}.AvailabilityZone}}'
+        )
 
     body = body.replace("$SUBNETS_TEMPLATES", "\n".join(subnet_templates))
     body = body.replace(
         "$SUBNETS_ROUTE_TABLE_ASSOCIATION", "\n".join(subnets_route_table_association),
     )
-    body = body.replace("$SUBNETS_REFERENCES", ",".join(subnets_references))
+    body = body.replace(
+        "$SUBNETS_WITH_AVAILABILITY_ZONES",
+        f'[{",".join(subnets_with_availability_zones)}]',
+    )
     body = body.replace("$ALLOWED_ORIGIN", shared_anyscale_conf.ANYSCALE_HOST)
 
     return body
@@ -877,7 +801,7 @@ def get_latest_ray_version():
     try:
         response = requests.get("https://pypi.org/pypi/ray/json")
         latest_version = response.json()["info"]["version"]
-    except Exception as e:
+    except Exception as e:  # noqa: BLE001
         log.debug(
             f"Unable to get latest Ray version from https://pypi.org/pypi/ray/json {str(e)}"
         )
@@ -887,12 +811,6 @@ def get_latest_ray_version():
 
 def get_ray_and_py_version_for_default_cluster_env() -> Tuple[str, str]:
     py_version = "".join(str(x) for x in sys.version_info[0:2])
-    if py_version not in ["36", "37", "38"]:
-        raise ValueError(
-            "No default cluster env for python version {}. Please use a version of python between 3.6 and 3.8.".format(
-                py_version
-            )
-        )
     try:
         import ray
 
@@ -951,68 +869,93 @@ def populate_dict_with_workspace_config_if_exists(
     If the job is submitted from the workspace, we populate values from the workspace
     if the config doesn't specify them, such as cluster env and compute config.
     """
-    if "ANYSCALE_EXPERIMENTAL_WORKSPACE_ID" in os.environ:
-        if "ANYSCALE_SESSION_ID" in os.environ:
-            cluster = anyscale_api_client.get_cluster(
-                os.environ["ANYSCALE_SESSION_ID"]
-            ).result
-            # If the job configs are not specified, infer them from the workspace:
-            if "build_id" not in config_dict and "cluster_env" not in config_dict:
-                config_dict["build_id"] = cluster.cluster_environment_build_id
-            if "project_id" not in config_dict:
-                config_dict["project_id"] = cluster.project_id
-            if (
-                "compute_config" not in config_dict
-                and "compute_config_id" not in config_dict
-            ):
-                config_dict["compute_config_id"] = cluster.cluster_compute_id
+    if (
+        "ANYSCALE_EXPERIMENTAL_WORKSPACE_ID" in os.environ
+        and "ANYSCALE_SESSION_ID" in os.environ
+    ):
+        cluster = anyscale_api_client.get_cluster(
+            os.environ["ANYSCALE_SESSION_ID"]
+        ).result
+        # If the job configs are not specified, infer them from the workspace:
+        if "build_id" not in config_dict and "cluster_env" not in config_dict:
+            config_dict["build_id"] = cluster.cluster_environment_build_id
+        if "project_id" not in config_dict:
+            config_dict["project_id"] = cluster.project_id
+        if (
+            "compute_config" not in config_dict
+            and "compute_config_id" not in config_dict
+        ):
+            config_dict["compute_config_id"] = cluster.cluster_compute_id
     return config_dict
 
 
-def get_allow_actions_from_policy_document(policy_document: Dict[Any, Any]) -> set:
-    allow_actions = {
+def filter_actions_from_policy_document(
+    policy_document: Dict[Any, Any], action_filter: Callable[[Dict], bool] = None
+) -> set:
+    if action_filter is None:
+        action_filter = lambda statement: statement["Effect"] == "Allow"  # noqa: E731
+    return {
         action
-        for statement in (
-            policy_document["Statement"]
-            if isinstance(policy_document["Statement"], list)
-            else [policy_document["Statement"]]
-        )
-        for action in (
-            statement["Action"]
-            if isinstance(statement["Action"], list)
-            else [statement["Action"]]
-        )
-        if statement["Effect"] == "Allow"
+        for statement in _coerce_to_list(policy_document["Statement"])
+        for action in _coerce_to_list(statement["Action"])
+        if action_filter(statement)
     }
-    return allow_actions
+
+
+def filter_actions_associated_with_role(
+    boto3_session: boto3.Session,
+    role: Boto3Resource,
+    action_filter: Callable[[Dict], bool] = None,
+) -> Set[str]:
+    iam = boto3_session.resource("iam")
+    attached_policy_documents = [
+        iam.PolicyVersion(policy.arn, policy.default_version_id).document
+        for policy in role.attached_policies.all()  # type: ignore
+    ]
+
+    role_policy_documents = [policy.policy_document for policy in role.policies.all()]  # type: ignore
+
+    list_of_allow_actions_sets = [
+        filter_actions_from_policy_document(policy_document, action_filter)
+        for policy_document in role_policy_documents + attached_policy_documents
+    ]
+    return (
+        set.union(*list_of_allow_actions_sets) if list_of_allow_actions_sets else set()
+    )
 
 
 def contains_control_plane_role(
-    assume_role_policy_document: Dict[Any, Any], anyscale_aws_account: str
+    assume_role_policy_document: Dict[str, Any], anyscale_aws_account: str
 ) -> bool:
-    for statement in (
-        assume_role_policy_document["Statement"]
-        if isinstance(assume_role_policy_document["Statement"], list)
-        else [assume_role_policy_document["Statement"]]
-    ):
+    def action_filter(statement: dict):
+        # Ensure it is `Allow` & `sts:AssumeRole`
         if (
-            statement["Effect"] == "Allow"
-            and statement.get("Action") == "sts:AssumeRole"
+            statement["Effect"] != "Allow"
+            or statement.get("Action") != "sts:AssumeRole"
         ):
-            for principal in (
-                statement["Principal"]
-                if isinstance(statement["Principal"], list)
-                else [statement["Principal"]]
-            ):
-                if "AWS" in principal:
-                    for aws_account in (
-                        principal["AWS"]
-                        if isinstance(principal["AWS"], list)
-                        else [principal["AWS"]]
-                    ):
-                        if aws_account in (
-                            f"arn:aws:iam::{anyscale_aws_account}:root",
-                            anyscale_aws_account,
-                        ):
-                            return True
-    return False
+            return False
+
+        expected_accounts = {
+            f"arn:aws:iam::{anyscale_aws_account}:root",
+            anyscale_aws_account,
+        }
+        return any(
+            aws_principal in expected_accounts
+            for aws_principal in _coerce_to_list(statement["Principal"].get("AWS"))
+        )
+
+    return (
+        len(
+            filter_actions_from_policy_document(
+                assume_role_policy_document, action_filter
+            )
+        )
+        > 0
+    )
+
+
+T = TypeVar("T")
+
+
+def _coerce_to_list(maybe_list: Union[T, List[T]]) -> List[T]:
+    return maybe_list if isinstance(maybe_list, list) else [maybe_list]
