@@ -80,7 +80,7 @@ def list_entities(
     list_function: Callable[..., Any],
     container_id: Optional[str] = None,
     filters: Optional[Dict[str, Any]] = None,
-    max: Optional[int] = None,
+    max: Optional[int] = None,  # noqa: A002
 ) -> List[Any]:
     """Convenience function to automatically handle paging tokens.
 
