@@ -45,7 +45,7 @@ class CommandPersister:
     def shutdown(self) -> None:
         self.is_active = False
 
-    async def persist_commands(self) -> None:
+    async def persist_commands(self) -> None:  # noqa: PLR0912
         while self.is_active:
             for session_command in self.running_commands.values():
                 self.upload_command_log(session_command)
@@ -95,7 +95,7 @@ class CommandPersister:
                 self.upload_command_log(cmd)
                 if cmd.finished:
                     del self.last_uploaded_time[cmd.scid]
-            except Exception as e:
+            except Exception as e:  # noqa: BLE001
                 with open(f"/tmp/{cmd.scid}.err", "a") as err_file:
                     err_file.write(f"{str(e)}\n")
             if len(self.queue) > 0:
