@@ -24,7 +24,7 @@ class TermState:
 
 class AnyscaleTermManager(NamedTermManager):  # type: ignore
     def __init__(self, command_persister: CommandPersister, **kwargs: Any) -> None:
-        super(AnyscaleTermManager, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         # For each terminal, record if it is a debug terminal
         # (keyed by terminal name).
         self.command_persister = command_persister
@@ -45,7 +45,7 @@ class AnyscaleTermManager(NamedTermManager):  # type: ignore
         is_debugger: bool = False,
         exec_command_id: Optional[str] = None,
     ) -> Any:
-        terminal = super(AnyscaleTermManager, self).get_terminal(term_name)
+        terminal = super().get_terminal(term_name)
         self.is_debugger[term_name] = is_debugger
         self.is_exec_command[term_name] = exec_command_id
         if term_name not in self.terminal_state:
@@ -54,7 +54,7 @@ class AnyscaleTermManager(NamedTermManager):  # type: ignore
         return terminal
 
     def new_named_terminal(self, **kwargs: Any) -> Tuple[str, Any]:
-        name, term = super(AnyscaleTermManager, self).new_named_terminal(**kwargs)
+        name, term = super().new_named_terminal(**kwargs)
         self.is_debugger[name] = False
         self.is_exec_command[name] = None
         self.terminal_state[name] = TermState()
@@ -64,10 +64,10 @@ class AnyscaleTermManager(NamedTermManager):  # type: ignore
     def terminal_count(self, include_all: bool = False) -> int:
         return sum(
             self.is_exec_command[term_name] is None or include_all
-            for term_name in self.terminals.keys()
+            for term_name in self.terminals
         )
 
-    def pty_read(self, fd: Any, events: Any = None) -> None:
+    def pty_read(self, fd: Any, events: Any = None) -> None:  # noqa: ARG002
         """
         Called by the event loop when there is pty data ready to read.
 
@@ -129,7 +129,7 @@ class AnyscaleTermManager(NamedTermManager):  # type: ignore
 
 class AnyscaleTermSocket(TermSocket):  # type: ignore
     def initialize(self, **kwargs: Any) -> None:
-        super(AnyscaleTermSocket, self).initialize(**kwargs)
+        super().initialize(**kwargs)
 
     def check_origin(self, origin: Any) -> bool:
         if self.application.settings["deploy_environment"] == "test":
@@ -178,7 +178,7 @@ class TerminalListHandler(tornado.web.RequestHandler):
                 "debugger": term_manager.is_debugger[term_name],
                 "exec_command": term_manager.is_exec_command[term_name] is not None,
             }
-            for term_name in term_manager.terminals.keys()
+            for term_name in term_manager.terminals
             if (term_manager.is_exec_command[term_name] is None or include_all)
         ]
         if term_manager.terminal_count() == 0 and create_if_none:
@@ -272,9 +272,9 @@ def make_application(
     )
 
 
-def main(
+def main(  # noqa: PLR0913
     deploy_environment: str,
-    use_debugger: bool,
+    use_debugger: bool,  # noqa: ARG001
     cli_token: str,
     host: str,
     working_dir: str,
@@ -288,7 +288,7 @@ def main(
     )
     port = 8700
     application.listen(port, "localhost")
-    print("Listening on localhost:{}".format(port))
+    print(f"Listening on localhost:{port}")
     loop.spawn_callback(command_persister.persist_commands)
 
     try:
