@@ -42,7 +42,7 @@ def test_ensure_no_credentials_on_head_node_config(
     node_provider_class, provider_config = node_provider_class_and_config
     with patch.multiple(
         "ray.autoscaler._private.aws.node_provider",
-        make_ec2_client=Mock(return_value=None),
+        make_ec2_resource=Mock(return_value=None),
     ), patch.multiple(
         "anyscale.authenticate.AuthenticationBlock",
         _validate_api_client_auth=Mock(),
@@ -71,7 +71,7 @@ def test_create_node_return() -> None:
     provider_config = {"region": "us-west-2", "cache_stopped_nodes": False}
     with patch.multiple(
         "ray.autoscaler._private.aws.node_provider",
-        make_ec2_client=Mock(return_value=mock_ec2_client),
+        make_ec2_resource=Mock(return_value=mock_ec2_client),
     ):
         node_provider = AnyscaleAWSNodeProvider(provider_config, "fake_cluster_name")
         created_node = node_provider.create_node({"SubnetIds": "123"}, tags={}, count=1)
