@@ -16,7 +16,7 @@ from anyscale.sdk.anyscale_client.models import ListResponseMetadata
 from anyscale.sdk.anyscale_client.models.cloud_list_response import CloudListResponse
 
 
-@pytest.fixture
+@pytest.fixture()
 def mock_prepare_cluster_block():
     with patch.multiple(
         "anyscale.connect_utils.prepare_cluster.PrepareClusterBlock",
@@ -652,7 +652,7 @@ def test_validate_new_cluster_compute_and_env_match_existing_cluster(
     )
     if (
         not build_matches
-        or not not cluster_compute_matches
+        or cluster_compute_matches
         or not allow_public_internet_traffic_matches
     ):
         assert observed_result
