@@ -112,20 +112,6 @@ def mock_role_with_external_id() -> Mock:
     )
 
 
-def test_setup_cloud_aws() -> None:
-    with patch.object(
-        CloudController, "setup_aws", return_value=None
-    ) as mock_setup_aws:
-        cloud_controller = CloudController()
-        cloud_controller.setup_cloud(
-            provider="aws", region=None, name="test-aws", yes=False,
-        )
-
-        mock_setup_aws.assert_called_once_with(
-            region="us-west-2", name="test-aws", yes=False
-        )
-
-
 def test_setup_cloud_gcp() -> None:
     mock_launch_gcp_cloud_setup = Mock(return_value=None)
     with patch.multiple(
