@@ -95,9 +95,10 @@ def example_compute_config() -> Dict[str, Any]:
 def compute_config_with_default_fields(
     write_compute_config: Dict[str, Any]
 ) -> Dict[str, Any]:
-    default_fields = dict(
-        idle_termination_minutes=IDLE_TIMEOUT_DEFAULT_MINUTES, region=None,
-    )
+    default_fields = {
+        "idle_termination_minutes": IDLE_TIMEOUT_DEFAULT_MINUTES,
+        "region": None,
+    }
     return dict(**write_compute_config, **default_fields)
 
 
@@ -168,7 +169,7 @@ def test_create(
 def test_delete(
     mock_auth_api_client,
     cluster_compute_name: Optional[str],
-    id: Optional[str],
+    id: Optional[str],  # noqa: A002
     cluster_compute_test_data: ClusterCompute,
 ) -> None:
     cluster_compute_controller = ClusterComputeController()
