@@ -22,7 +22,7 @@ from anyscale.sdk.anyscale_client import StartClusterOptions
 @pytest.mark.parametrize("cluster_compute", [Mock(id="compute1"), Mock(id="compute2")])
 @pytest.mark.parametrize("passed_cluster_env", [True, False])
 @pytest.mark.parametrize("passed_cluster_compute", [True, False])
-def test_check_needs_start(
+def test_check_needs_start(  # noqa: PLR0913
     cluster: Mock,
     build: Mock,
     cluster_compute: Mock,
