@@ -7,7 +7,7 @@ from types import SimpleNamespace
 from typing import Any, Dict, List, Optional, Tuple, Type
 from unittest.mock import MagicMock, Mock, mock_open, patch
 
-from aiohttp import ClientSession, WSMsgType
+from aiohttp import ClientResponseError, ClientSession, WSMsgType
 from asynctest import CoroutineMock
 import click
 import pytest
@@ -48,7 +48,7 @@ class FakeLogger(LogsLogger):
         print(msg)
 
 
-@pytest.fixture
+@pytest.fixture()
 def test_directory():
     with tempfile.TemporaryDirectory() as tmp_dir:
         path = Path(tmp_dir)
@@ -72,7 +72,7 @@ def test_directory():
         os.chdir(old_dir)
 
 
-@pytest.fixture
+@pytest.fixture()
 def patch_jobs_anyscale_api_client(base_mock_anyscale_api_client: Mock):
     base_mock_anyscale_api_client.get_cluster_environment_build = Mock(
         return_value=Mock(result=Mock(status="succeeded"))
@@ -223,7 +223,7 @@ def test_submit_job(
             description="mock_description",
             project_id=final_project_id,
             workspace_id=None,
-            config=ProductionJobConfig(
+            config=ProductionJobConfig(  # noqa: PIE804
                 **{
                     "entrypoint": "mock_entrypoint",
                     "build_id": "mock_build_id",
@@ -260,7 +260,7 @@ def test_submit_job(
     "passed_service_id", [False, True]
 )  # Whether `job_id` is id of job or service
 @pytest.mark.parametrize("include_archived", [False, True])
-def test_list_jobs(
+def test_list_jobs(  # noqa: PLR0913
     mock_auth_api_client,
     include_all_users: bool,
     name: Optional[str],
@@ -357,7 +357,10 @@ def test_list_jobs(
 @pytest.mark.parametrize("name", ["mock_job_name", None])
 @pytest.mark.parametrize("id", ["mock_job_id", None])
 def test_terminate_job(
-    mock_auth_api_client, name: Optional[str], id: Optional[str], is_service: bool,
+    mock_auth_api_client,
+    name: Optional[str],
+    id: Optional[str],  # noqa: A002
+    is_service: bool,
 ) -> None:
     job_controller = JobController()
     job_controller.api_client.list_decorated_jobs_api_v2_decorated_ha_jobs_get = Mock(
@@ -386,7 +389,10 @@ def test_terminate_job(
 @pytest.mark.parametrize("name", ["mock_job_name", None])
 @pytest.mark.parametrize("id", ["mock_job_id", None])
 def test_archive_job(
-    mock_auth_api_client, name: Optional[str], id: Optional[str], is_service: bool,
+    mock_auth_api_client,
+    name: Optional[str],
+    id: Optional[str],  # noqa: A002
+    is_service: bool,
 ) -> None:
     job_controller = JobController()
     job_controller.api_client.list_decorated_jobs_api_v2_decorated_ha_jobs_get = Mock(
@@ -746,7 +752,7 @@ def fake_log_provider(mode: str) -> Type[LogProvider]:
         def close(self):
             pass
 
-        def query(
+        def query(  # noqa: PLR0911
             self, start_timestamp_ns: int, end_timestamp_ns: int
         ) -> List[Tuple[int, str]]:
             # For typing
@@ -814,10 +820,10 @@ class MockAsyncContextManagerReturnValue(MagicMock):
         return None
 
 
+# This also indirectly tests _read_websocket()
 def test_stream_logs(mock_auth_api_client) -> None:
     job_controller = JobController(log=FakeLogger())
     prodjob_id = "prodjob_123"
-    job_controller._resolve_job_object = Mock(return_value=Mock(id=prodjob_id))  # type: ignore
     job_id = "job_123"
     job_controller._wait_for_a_job_run = Mock(return_value=job_id)  # type: ignore
     job_controller.api_client = Mock(
@@ -826,6 +832,7 @@ def test_stream_logs(mock_auth_api_client) -> None:
         )
     )
     mock_websocket = MockAsyncContextManagerReturnValue()
+    # Test logs successfully streamed
     mock_websocket.receive = CoroutineMock(
         side_effect=[
             Mock(type=WSMsgType.TEXT, data=" abc "),
@@ -836,14 +843,42 @@ def test_stream_logs(mock_auth_api_client) -> None:
     with patch.object(
         ClientSession, ClientSession.ws_connect.__name__, return_value=mock_websocket
     ):
-        job_controller.stream_logs(job_id)
-    job_controller._resolve_job_object.assert_called_once_with(job_id, None)
+        job_controller.stream_logs(prodjob_id)
     job_controller._wait_for_a_job_run.assert_called_once_with(prodjob_id)
     job_controller.api_client.get_job_logs_stream_api_v2_logs_job_logs_stream_job_id_get.assert_called_once_with(
         job_id=job_id
     )
     assert mock_websocket.receive.call_count == 3
     assert job_controller.log.logs == ["abc"]  # type: ignore
+    # Test job does not exist
+    mock_job_run_not_found_exception = ClientResponseError(
+        request_info=Mock(), history=Mock(), status=404
+    )
+    mock_websocket.receive = CoroutineMock(side_effect=mock_job_run_not_found_exception)
+    with patch.object(
+        ClientSession, ClientSession.ws_connect.__name__, return_value=mock_websocket
+    ), pytest.raises(
+        Exception  # noqa: PT011
+    ) as e:
+        job_controller.stream_logs(job_id)
+    assert "Job run job_123 not found in cluster for current running job." in str(e)
+
+    # Test other exception thrown
+    mock_other_exception = ClientResponseError(
+        request_info=Mock(), history=Mock(), status=400
+    )
+    mock_websocket.receive = CoroutineMock(side_effect=mock_other_exception)
+    with patch.object(
+        ClientSession, ClientSession.ws_connect.__name__, return_value=mock_websocket
+    ), pytest.raises(
+        Exception  # noqa: PT011
+    ) as e:
+        job_controller.stream_logs(job_id)
+
+    # Test keyboard interrupt handled
+    job_controller._read_websocket = CoroutineMock(side_effect=KeyboardInterrupt)  # type: ignore
+    with patch.object(ClientSession, ClientSession.ws_connect.__name__):
+        job_controller.stream_logs(job_id)
 
 
 @pytest.mark.parametrize("cluster_id_found_in_last_job_run", [True, False])
@@ -853,7 +888,6 @@ def test_download_logs(
     prodjob_id = "prodjob_123"
     cluster_id = "session_123"
     job_controller = JobController(log=FakeLogger())
-    job_controller._resolve_job_object = Mock(return_value=Mock(id=prodjob_id))  # type: ignore
     if cluster_id_found_in_last_job_run:
         mock_prod_job = Mock(last_job_run=Mock(cluster_id=cluster_id), state=Mock())
     else:
@@ -864,7 +898,6 @@ def test_download_logs(
     mock_download_logs = Mock()
     job_controller.logs_controller.download_logs = mock_download_logs  # type: ignore
     job_controller.download_logs(prodjob_id)
-    job_controller._resolve_job_object.assert_called_once_with(prodjob_id, None)
     job_controller._get_latest_job_run.assert_called_once_with(prodjob_id)
     mock_download_logs.assert_called_once_with(
         filter=LogFilter(cluster_id=cluster_id, glob="/job-driver-"),
@@ -876,7 +909,7 @@ def test_download_logs(
     "mode",
     ["empty", "single", "paginate", "follow", "flush_success", "wait_for_running"],
 )
-def test_job_logs(mock_auth_api_client, capsys, mode) -> None:
+def test_loki_logs(mock_auth_api_client, capsys, mode) -> None:
     job_controller = JobController(log=FakeLogger())
     fake_log_provider_cls = fake_log_provider(mode)
 
@@ -908,35 +941,34 @@ def test_job_logs(mock_auth_api_client, capsys, mode) -> None:
         )
     )
 
-    def do_logs() -> None:
-        job_controller.logs(
+    def do_loki_logs() -> None:
+        job_controller.loki_logs(
             job_id="mock_job_id",
-            job_name=None,
             should_follow=mode in ["follow", "flush_success", "wait_for_running"],
             log_provider_cls=fake_log_provider_cls,
         )
 
     if mode == "empty":
-        do_logs()
+        do_loki_logs()
         assert capsys.readouterr().out.strip() == ""
     elif mode == "single":
-        do_logs()
+        do_loki_logs()
         assert capsys.readouterr().out.strip() == "start\nend"
     elif mode == "paginate":
-        do_logs()
+        do_loki_logs()
         assert capsys.readouterr().out.strip() == "entry-1\nentry-2"
     elif mode == "follow":
         with pytest.raises(IndexError):  # controller should continue to query
-            do_logs()
+            do_loki_logs()
         out = capsys.readouterr().out.strip()
         assert out == "finally"
     elif mode == "flush_success":
-        do_logs()
+        do_loki_logs()
         out = capsys.readouterr().out.strip()
         assert out == "finally"
     elif mode == "wait_for_running":
         with pytest.raises(IndexError):  # controller should continue to query
-            do_logs()
+            do_loki_logs()
         assert job_controller._wait_for_job_running.call_count == 1
         out = capsys.readouterr().out.strip()
         assert out == "finally"
@@ -944,6 +976,59 @@ def test_job_logs(mock_auth_api_client, capsys, mode) -> None:
         raise Exception("unreachable")
 
 
+def test_logs(mock_auth_api_client) -> None:
+    job_controller = JobController()
+    prod_job_id = "prodjob_123"
+    job_controller._resolve_job_object = Mock(return_value=Mock(id=prod_job_id))  # type: ignore
+    job_controller.stream_logs = mock_stream_logs = Mock()  # type: ignore
+    job_controller.download_logs = mock_download_logs = Mock()  # type: ignore
+    job_controller.loki_logs = mock_loki_logs = Mock()  # type: ignore
+
+    def reset_mocks() -> None:
+        for mock in [mock_download_logs, mock_stream_logs, mock_loki_logs]:
+            mock.reset_mock()
+
+    def do_assertions(
+        called: Optional[List[Mock]] = None, not_called: Optional[List[Mock]] = None
+    ) -> None:
+        if not called:
+            called = []
+        if not not_called:
+            not_called = []
+        for mock in called:
+            mock.assert_called_once_with(prod_job_id)
+        for mock in not_called:
+            mock.assert_not_called()
+
+    # Test download logs by default
+    job_controller.logs(job_id=prod_job_id)
+    do_assertions(
+        called=[mock_download_logs], not_called=[mock_stream_logs, mock_loki_logs]
+    )
+
+    # Test stream logs if follow specified
+    reset_mocks()
+    job_controller.logs(job_id=prod_job_id, should_follow=True)
+    do_assertions(
+        called=[mock_stream_logs], not_called=[mock_download_logs, mock_loki_logs]
+    )
+
+    # Test download if stream fails
+    reset_mocks()
+    job_controller.stream_logs.side_effect = Exception
+    job_controller.logs(job_id=prod_job_id, should_follow=True)
+    do_assertions(
+        called=[mock_stream_logs, mock_download_logs], not_called=[mock_loki_logs]
+    )
+
+    # Test loki logs if both stream and download fails
+    reset_mocks()
+    job_controller.download_logs.side_effect = Exception
+    job_controller.logs(job_id=prod_job_id, should_follow=True)
+    do_assertions(called=[mock_stream_logs, mock_download_logs])
+    mock_loki_logs.assert_called_once_with(prod_job_id, True)
+
+
 def test_wait_for_job_running(mock_auth_api_client) -> None:
     job_controller = JobController(log=FakeLogger())
     i = 0
@@ -1023,7 +1108,10 @@ def test_job_wait(mock_auth_api_client) -> None:
     job_controller.log = MagicMock()
     job_controller.log.spinner.return_value.__enter__.return_value = spin_mock
 
-    def get_job_state(states_to_return, mock_state=Mock()):
+    def get_job_state(states_to_return, mock_state=None):
+        if mock_state is None:
+            mock_state = Mock()
+
         mock_state.i = 0
 
         def g(*args, **kwargs):
