@@ -30,7 +30,7 @@ PIP_LIST = ["requests==1.0.0", "pip-install-test"]
 GS_BUCKET_NAME = "anyscale-bk-e2e-test-job"
 
 
-@pytest.fixture
+@pytest.fixture()
 def test_directory():
     with tempfile.TemporaryDirectory() as tmp_dir:
         path = Path(tmp_dir)
@@ -52,7 +52,7 @@ def test_directory():
 
 
 @pytest.mark.parametrize(
-    "upload_path,expected_output",
+    ("upload_path", "expected_output"),
     [
         ("gs://bucket", "file.zip"),
         ("gs://bucket/path", "path/file.zip"),
@@ -152,9 +152,10 @@ class TestUploadLocalWorkingDir:
 
         # Download the zip file from the remote bucket
         local_zip_file = "test_job_controller_working_dir.zip"
-        with smart_open.open(remote_uri, "rb") as package_zip:
-            with open(local_zip_file, "wb") as fin:
-                fin.write(package_zip.read())
+        with smart_open.open(remote_uri, "rb") as package_zip, open(
+            local_zip_file, "wb"
+        ) as fin:
+            fin.write(package_zip.read())
 
         # Check the downloaded zip is the same directory that was uploaded
         unzip_target_dir = os.path.join("unzip_target_dir")
