@@ -33,15 +33,15 @@ def test_block():
 
     # Test error raised when opening two blocks without closing first
     log.open_block(block_label="block1")
-    with pytest.raises(Exception):
+    with pytest.raises(Exception):  # noqa: PT011
         log.open_block(block_label="block2")
 
     # Test error raised when closing block that wasn't opened
-    with pytest.raises(Exception):
+    with pytest.raises(Exception):  # noqa: PT011
         log.close_block(block_label="block2")
 
     # Test error raised when writing to block that wasn't opened
-    with pytest.raises(Exception):
+    with pytest.raises(Exception):  # noqa: PT011
         log.info("msg2", block_label="block2")
 
 
