@@ -17,7 +17,7 @@ from anyscale.util import (
 
 
 @pytest.mark.parametrize(
-    "working_dir, remote_location, result",  # type: ignore
+    ("working_dir", "remote_location", "result"),  # type: ignore
     [
         ("test-working-dir", "/a/b/c", "/a/b/c"),
         ("~/test-working-dir", "/root/a/b/c", "~/a/b/c"),
