@@ -313,7 +313,7 @@ def _create_cloudformation_stack(region: str) -> str:
 @pytest.mark.parametrize(
     ("vpc_exists", "vpc_cidr_block", "expected_result", "expected_log_message"),
     [
-        pytest.param(False, "0.0.0.0/0", False, r"does not exist."),
+        pytest.param(False, "0.0.0.0/0", False, r"Could not find"),
         # Happy sizes
         pytest.param(True, "0.0.0.0/16", True, r"verification succeeded."),
         pytest.param(True, "0.0.0.0/20", True, r"verification succeeded."),
@@ -615,7 +615,7 @@ def test_verify_aws_iam_roles_instance_profiles():
         pytest.param(
             False, False, False, r"Missing security group IDs", id="SG-Does-Not-Exist"
         ),
-        pytest.param(True, False, False, r"does not exist", id="SG-Exists-Not-Valid"),
+        pytest.param(True, False, False, r"Could not find", id="SG-Exists-Not-Valid"),
         pytest.param(True, True, True, None, id="SG-Exists-And-Valid"),
     ],
 )
@@ -751,7 +751,7 @@ def test_verify_aws_security_groups_inbound_permissions(
     ],
 )
 @pytest.mark.parametrize(
-    "cors_rules,expected_warning_call_count",
+    ("cors_rules", "expected_warning_call_count"),
     [
         pytest.param(
             [
