@@ -53,7 +53,7 @@ def test_configure_ssh_key(base_mock_api_client: Mock) -> None:
         output_config = _configure_ssh_key(
             input_config, "session_id", base_mock_api_client, directory
         )
-        with open(os.path.join(directory, "SSH_KEY.pem"), "r") as f:
+        with open(os.path.join(directory, "SSH_KEY.pem")) as f:
             assert f.read() == "PRIVATE_KEY"
     assert "SSH_KEY" in output_config["auth"]["ssh_private_key"]
 
