@@ -31,7 +31,7 @@ from frontend.cli.anyscale.aws_iam_policies import (
 )
 from frontend.cli.anyscale.util import (
     contains_control_plane_role,
-    get_allow_actions_from_policy_document,
+    filter_actions_from_policy_document,
 )
 
 
@@ -106,7 +106,7 @@ STATEMENT_TEMPLATE = {
 
 
 @pytest.mark.parametrize(
-    "statement_policy,expected_conditions",
+    ("statement_policy", "expected_conditions"),
     [
         pytest.param(
             [STATEMENT_TEMPLATE],
@@ -172,7 +172,7 @@ def test_update_external_ids_for_policy(statement_policy, expected_conditions):
 
 
 @pytest.mark.parametrize(
-    "image_name, expected, exception_substr",
+    ("image_name", "expected", "exception_substr"),
     [
         ("anyscale/ray-ml:1.11.1-py38-gpu", ("py38", "1.11.1"), None),
         ("anyscale/ray:1.12-py37-cpu", ("py37", "1.12"), None),
@@ -191,7 +191,7 @@ def test_extract_versions_from_image_name(image_name, expected, exception_substr
 
 
 @pytest.mark.parametrize(
-    "ray_version, expected, exception_substr",
+    ("ray_version", "expected", "exception_substr"),
     [
         ("1.12", (1, 12), None),
         ("0.0", (0, 0), None),
@@ -204,7 +204,7 @@ def test_extract_versions_from_image_name(image_name, expected, exception_substr
 )
 def test_ray_version_major_minor(ray_version, expected, exception_substr):
     if exception_substr is not None:
-        with pytest.raises(Exception) as exc_info:
+        with pytest.raises(Exception) as exc_info:  # noqa: PT011
             _ray_version_major_minor(ray_version)
         assert exception_substr in str(exc_info.value)
     else:
@@ -213,7 +213,7 @@ def test_ray_version_major_minor(ray_version, expected, exception_substr):
 
 
 @pytest.mark.parametrize(
-    "python_version, exception_substr",
+    ("python_version", "exception_substr"),
     [
         ("py36", None),
         ("py37", None),
@@ -227,7 +227,7 @@ def test_ray_version_major_minor(ray_version, expected, exception_substr):
 )
 def test_python_version_major_minor(python_version, exception_substr):
     if exception_substr is not None:
-        with pytest.raises(Exception) as exc_info:
+        with pytest.raises(Exception) as exc_info:  # noqa: PT011
             _check_python_version(python_version)
         assert exception_substr in str(exc_info.value)
     else:
@@ -290,7 +290,7 @@ def test_prepare_cloudformation_template():
     assert (
         cfn_template
         == """Description: This tempalte creates the resources necessary for anyscale clouds to function.
-
+Transform: AWS::LanguageExtensions
 Parameters:
   EnvironmentName:
     Description: Anyscale deploy environment. Used in resource names and tags.
@@ -327,8 +327,8 @@ Parameters:
     Type: String
     Default: "anyscale-security-group"
 
-  ClusterNodeIAMRoleArn:
-    Description: ARN of the data plane cluster IAM role
+  ClusterNodeIAMRoleName:
+    Description: Name of the data plane cluster IAM role
     Type: String
 
 Resources:
@@ -534,6 +534,7 @@ Resources:
   S3Bucket:
     Type: AWS::S3::Bucket
     Description: Anyscale managed s3 bucket.
+    DeletionPolicy: Retain
     Properties:
       BucketName: !Sub anyscale-${EnvironmentName}-data-${AWS::StackName}
       CorsConfiguration:
@@ -575,7 +576,9 @@ Resources:
               - !Sub arn:aws:s3:::${S3Bucket}/*
             Principal:
               AWS:
-                - !Ref ClusterNodeIAMRoleArn
+                - !GetAtt
+                  - nodeRole
+                  - Arn
                 - !GetAtt
                   - customerRole
                   - Arn
@@ -628,6 +631,35 @@ Resources:
         - Key: anyscale-deploy-environment
           Value: !Ref EnvironmentName
 
+  nodeRole:
+    Type: 'AWS::IAM::Role'
+    Properties:
+      RoleName: !Ref ClusterNodeIAMRoleName
+      AssumeRolePolicyDocument:
+        Statement:
+          - Action: 'sts:AssumeRole'
+            Effect: Allow
+            Principal:
+              Service: ec2.amazonaws.com
+            Sid: 'Allow'
+        Version: 2012-10-17
+      Path: /
+      Tags:
+        - Key: Name
+          Value: !Ref AWS::StackName
+        - Key: anyscale-cloud-id
+          Value: !Ref CloudID
+        - Key: anyscale-deploy-environment
+          Value: !Ref EnvironmentName
+
+  IamInstanceProfile:
+    Type: AWS::IAM::InstanceProfile
+    Properties:
+      InstanceProfileName: !Ref nodeRole
+      Path: /
+      Roles:
+        - !Ref nodeRole
+
   IAMPermissionEC2SteadyState:
     Type: AWS::IAM::Policy
     Properties:
@@ -649,9 +681,11 @@ Outputs:
     Description: A reference to the created VPC
     Value: !Ref VPC
 
-  PublicSubnets:
-    Description: A list of the public subnets
-    Value: !Join [",", [!Ref Subnet0,!Ref Subnet1,!Ref Subnet2,!Ref Subnet3,!Ref Subnet4,!Ref Subnet5]]
+  SubnetsWithAvailabilityZones:
+    Description: A list of the subnets with their availability zones
+    Value:
+      Fn::ToJsonString:
+        [{"subnet_id": !Ref Subnet0, "availability_zone": !GetAtt Subnet0.AvailabilityZone},{"subnet_id": !Ref Subnet1, "availability_zone": !GetAtt Subnet1.AvailabilityZone},{"subnet_id": !Ref Subnet2, "availability_zone": !GetAtt Subnet2.AvailabilityZone},{"subnet_id": !Ref Subnet3, "availability_zone": !GetAtt Subnet3.AvailabilityZone},{"subnet_id": !Ref Subnet4, "availability_zone": !GetAtt Subnet4.AvailabilityZone},{"subnet_id": !Ref Subnet5, "availability_zone": !GetAtt Subnet5.AvailabilityZone}]
 
   AnyscaleSecurityGroup:
     Description: Anyscale Security group
@@ -676,6 +710,12 @@ Outputs:
     Value: !GetAtt
       - customerRole
       - Arn
+
+  NodeIamRole:
+    Description: ARN of the node IAM role
+    Value: !GetAtt
+      - nodeRole
+      - Arn
 """
     )
 
@@ -700,7 +740,9 @@ def test_get_latest_ray_version(request_output):
     mock_get.assert_called_once_with("https://pypi.org/pypi/ray/json")
 
 
-@pytest.mark.parametrize("pyversion_param", [["3", "6"], ["3", "7"], ["3", "8"]])
+@pytest.mark.parametrize(
+    "pyversion_param", [["3", "6"], ["3", "7"], ["3", "8"], ["3", "9"]]
+)
 def test_get_ray_and_py_version_for_default_cluster_env(pyversion_param):
     # TODO(nikita): This test should be run in an environment that has Ray
     # installed, and one that doesn't have Ray installed. The
@@ -803,7 +845,7 @@ def test_populate_dict_method_in_workspace(config_dict: Dict[str, Any]):
 
 
 @pytest.mark.parametrize(
-    "policy_document,expected_actions",
+    ("policy_document", "expected_actions"),
     [
         pytest.param(
             ANYSCALE_IAM_PERMISSIONS_EC2_STEADY_STATE,
@@ -885,13 +927,13 @@ def test_populate_dict_method_in_workspace(config_dict: Dict[str, Any]):
         ),
     ],
 )
-def test_get_allow_actions_from_policy_document(policy_document, expected_actions):
-    actions = get_allow_actions_from_policy_document(policy_document=policy_document)
+def test_filter_actions_from_policy_document(policy_document, expected_actions):
+    actions = filter_actions_from_policy_document(policy_document=policy_document)
     assert actions == expected_actions
 
 
 @pytest.mark.parametrize(
-    "assume_role_policy_document,expected_result",
+    ("assume_role_policy_document", "expected_result"),
     [
         pytest.param(
             {
@@ -944,6 +986,18 @@ def test_get_allow_actions_from_policy_document(policy_document, expected_action
             },
             False,
         ),
+        pytest.param(
+            {
+                "Statement": [
+                    {
+                        "Effect": "Allow",
+                        "Principal": {"Service": "arn:aws:iam::000000:root"},
+                        "Action": "sts:AssumeRole",
+                    },
+                ],
+            },
+            False,
+        ),
     ],
 )
 def test_contains_control_plane_role(assume_role_policy_document, expected_result):
